 /** Trabajo Final de Grado. UOC.
  *  11 de junio del 2017
  *  Sergio Fernández Ferri
  *  Grado en Tecnología de las comunicaciones. Esp. Sistemas Telecomunicación.
  *  -------------------------------------------------------------------------
  *  Sistema Domótico Arduino con RF y portal WEB.
  *  -------------------------------------------------------------------------
  *  Código a implementar en el nodo 00 (master) conectado al router con
  *  acceso a internet.
  *  -------------------------------------------------------------------------
  */
#include <SPI.h>
#include <Ethernet2.h> // Librería Ethernet2 en lugar de Ethernet
#include <AESLib.h>
#include "RF24.h"
#include "RF24Network.h"
#include <EEPROM.h>

// Número primo y coprimo (generador) para el algoritmo Diffie-Hellman
const uint32_t PRIMO =2147483647;
const uint16_t GENERADOR =16807;
uint32_t b=0;
uint32_t momentoUltimaActividad; // Variable para almacenar el momento de la última actividad autorizada.
uint32_t tiempoInactivo=0;  // Variable para controla el tiempo de inactividad y desautorizar el acceso.
#define RESETPIN 2  // Pin para reseteo contraseña y usario (admin:1234)
String USER="";
String PASS="";
#define MAX_TIEMPO_INACTIVO 60000  // en ms 60000 = 60 s. Máximo tiempo permitido de inactividad. Si se supera se desautoriza.
uint32_t tiempoDDNS; // variable para almacenar el timepo transcurrido para actualizar IP en DDNS 
#define MAXTIEMPOACTUALIZACIONIP 60000 // Cada cuantos ms actualizamos el valor de nuestra IP en el servido DDNS No-IP
byte ipPublica[4]; // Array para almacenar el valor de nuestra IP pública en formato IPV4

/***** Creamos un obtjeto RF24 para gestionar el módulo nRFL2401 *****/
RF24 radio(53,48);  // Indicamos que pines del Mega actuarán como Chip Enable y Chip Select en el nRFL2401
RF24Network network(radio); // Creamos un objeto RF24Network pasándole como parámetro el objeto del módulo RF.
const uint16_t this_node=00;  // Número de nodo (00 es el maestro de la red o nodo central).

#define refreshPage 8  // Refresco de páginas (en segundos)

String lectura;  // Variable para almacenar la lectura del cliente WEB (el navegador que se conecte al servidor).

const String elementos[7] PROGMEM = {  // Elementos o funciones que podemos controlar o gestionar con el sistema
  "Luces","Temperatura","Persianas","AACC","Puertas","Consumos","Cambiar datos acceso"
};
#define numElementos 7  // Número de elementos. Para fijar las iteraciones de los bucles.

const String estancias[9] PROGMEM = { // Estancias dónde hay elementos a gestionar.
  "Salon","Entrada","Cocina","Dormitorio 1","Dormitorio 2","Dormitorio 3","Dormitorio 4","Aseo","Garaje"
};
#define numEstancias 8 // Número de estancias. Para fijar las iteraciones de los bucles.

#define PIN_LM35 0 // Pin al conectamos el LM35 para lectura de temperatura


/////////////////////////////////////////////////////////////////////////////////////////////////
// Mensajes entre nodos RF
/////////////////////////////////////////////////////////////////////////////////////////////////
// Tipo de consultas
#define ESTADO_LUZ 1001
#define ESTADO_PUERTA 1002
#define VER_CONSUMO 1003
#define VER_TEMPERATURA 1004
#define ESTADO_PERSIANA 1005
#define ESTADO_AACC 1006

// Tipo de acciones
#define APAGAR_LUZ 1201
#define ENCENDER_LUZ 1211

#define BAJAR_PERSIANA 1215
#define SUBIR_PERSIANA 1225

#define ENCEDER_AACC 1216
#define APAGAR_AACC 1226
#define SUBIR_TEMP_AACC 1236
#define BAJAR_TEMP_AACC 1246
#define FAN_AACC 1256

/////////////////////////////////////////////////////////////////////////////////////////////////
// Valores fijos que utilizaremos por comodidad para el manejo de las páginas WEB
/////////////////////////////////////////////////////////////////////////////////////////////////
#define REFRESCAR 1
#define LUCES 0
#define TEMPERATURA 1
#define PERSIANAS 2
#define AACC 3
#define PUERTAS 4
#define PAGINA_CONSUMO 5
#define CAMBIOPASSWORD 6
#define MENU 7
#define AUTORIZAR 8


/////////////////////////////////////////////////////////////////////////////////////////////////
// Configuracion Ethernet
/////////////////////////////////////////////////////////////////////////////////////////////////

// IP configuración manual 
//(Si queremos que la configuración la realice nuestro router mediante DHCP únicamente deberemos declarar la mac, el resto de variables no)
// Dado que deberemos tener acceso al microcontrolador deberemos tener una IP fija conocida para acceder a él.
byte ip[] = { 192, 168, 1, 177 };   // Seleccionar una IP libre
byte gateway[] = { 192, 168, 1, 1 }; // Indicar la puerta de enlace
byte subnet[] = {  255, 255, 255, 0 }; // Indicar la máscara de subred
byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED }; // Dirección MAC de nuestro Ethernet Shield


// Creamos una instancia del servidor escuchando en el puerto 80 (HTTP)
EthernetServer server = EthernetServer(80);

/////////////////////////////////////////////////////////////////////////////////////////////////
// Creamos una variable que conserve el estado de la atutorización mientras navegamos por las páginas WEB
/////////////////////////////////////////////////////////////////////////////////////////////////
boolean autorizado=false;

/////////////////////////////////////////////////////////////////////////////////////////////////
// Creamos clases para el manejo de los diferentes módulos virtuales.
/////////////////////////////////////////////////////////////////////////////////////////////////
// Clase modulo - Representa el acceso a un modulo externo (sensor de temperatura, de corriente, relé, control AACC mediante IR, etc...
class Modulo {
  protected :
    String Estancia;  // Contiene en que estancia está situado
    uint16_t NumeroNodo;  // Nos indica a que nodo RF de la red Ethernet RF está conectado (la dirección del nodo)
    
  public:
    Modulo(){}; // Constructor de la clase
    void setEstancia(String estancia); // Método para establecer la estancia
    void setNumeroNodo(uint16_t numeroNodo);  // Método para establecer el número del nodo
    //void setNodo(Nodo_RF24_Domo nodo);  
    String getEstancia();  // Método para obtener la estancia.
    uint16_t getNumeroNodo(); // Método para obtener el número de nodo.
    
};
void Modulo::setEstancia(String estancia) {Estancia=estancia;}
void Modulo::setNumeroNodo(uint16_t numeroNodo) {NumeroNodo=numeroNodo;}
String Modulo::getEstancia() {return Estancia;}
uint16_t Modulo::getNumeroNodo() {return NumeroNodo;}
/////////////////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////////////////////
// Clase Luz - hereda de la clase módulo
/////////////////////////////////////////////////////////////////////////////////////////////////
class Luz : public Modulo {
  private :
    boolean Estado; // Variable donde se almacenará el estado de la luz que el nodo RF nos envíe.

  public:
    //Luz();
    void setEstado(boolean estado); // Método para establecer el estado de la luz (encender o apagar). 
    void actualizar();  // Método para obtener el estado de la luz desde el módulo de RF correspondiente.
    boolean getEstado();  // Método que nos devuelve el valor del estado almacenado.
};

void Luz::setEstado(boolean estado) // Con este método daremos las órdenes correspondientes al modulo RF para que el nodo RF receptor encienda o apague la luz.
  {
    unsigned int solicitud;
    if (estado){
      solicitud = ENCENDER_LUZ;  // Creamos una variable para enviar la solicitud de encender
    } else {
      solicitud = APAGAR_LUZ;  // Creamos una variable para enviar la solicitud de apagar
    }
    // Si recibimos orden de encender la luz (estado=true)
     
      network.update(); // Actualizamos el estado de la red RF antes de comunicar.
      // Líneas de debug
      ////Serial.print("Encendiendo/Apagando la luz en estancia : "); 
      ////Serial.println(Estancia);
      ////Serial.print("A través del nodo : ");
      ////Serial.println(NumeroNodo, OCT);
     /**** Creamos la cabecera del paquete IP con dirección de destino NumeroNodo (el nodo correspondiente a la luz que queremos encender).
      *  Indicamos un tipo de mensaje 't' (126) para que la red gestione el ACK del nodo correspondiente.
      *  En caso contrario (no indicar un tipo de mensaje -> por defecto será el 0), o indicar un tipo de mensaje de entre el 1 o el 64, hará que la capa de red no 
      *  gestione los ACK de los nodos que no está conectados directamente al nodo master (nodos hijos de otros nodos), lo que conllevará un reconocimiento de entrega falso.
      */
     RF24NetworkHeader header(NumeroNodo, 't');  
      if (!network.write(header, &solicitud, sizeof(solicitud))) {  // Enviamos el paquete con los datos de la variable solicitud.
        ////Serial.println(F("Send fail"));  // Líneas debug
      
        } else {
        ////Serial.print(F("Send OK: ")); //Serial.print(solicitud);  //Serial.print(" al nodo : "); //Serial.println(NumeroNodo, OCT);
        Estado=estado;  // Si el envío se ha realizado correctamente modificamos el Estado.
        }
       
  }
    

void Luz::actualizar() {  // Con este método comunicaremos con el nodo RF correspondiente para obtener el estado de la luz.
  network.update();
  
  ////Serial.print("Pidiendo actualizacion al nodo : ");
  ////Serial.println(NumeroNodo, OCT);
  
  unsigned int solicitud = ESTADO_LUZ; // Solicitar actualización datos.


  RF24NetworkHeader header(NumeroNodo, 't'); //Indicamos un tipo de mensaje 't' (126) para que la red gestione el ACK del nodo correspondiente. En caso contrario 
  if (!network.write(header, &solicitud, sizeof(solicitud))) {
    ////Serial.println(F("Send fail"));
      
    } else {
      ////Serial.print(F("Send OK: ")); //Serial.print(solicitud);  //Serial.print(" al nodo : "); //Serial.println(NumeroNodo, OCT);
  }

  delay(40); // Damos tiempo a que el nodo pueda responder para leer los datos. El nodo RF ha de comprobar el estado de la luz y tarda un tiempo. Debemos leer la red después de que el nodo mande los datos.
  network.update(); // Actualizamos los datos de la red para obtener el paquete que envia el nodo RF.
  
  Estado=false; // Si no conseguimos comunicar y recibir el estado de la luz mostraremos que está apagada.
  
    while(network.available()){
    RF24NetworkHeader header;
    network.read(header, &solicitud, sizeof(unsigned long));  // Almacenamos el estado de la luz en la variable solicitud para no ocupar más memoria.
    if (header.from_node==NumeroNodo) {  // Si el paquete recibido corresponde a este nodo
        // Líneas debug
        
        //String cabecera=header.toString();
        ////Serial.println(cabecera);
        
        ////Serial.print("Recibimos : ");
        ////Serial.print(solicitud);
        ////Serial.print(" del nodo :");
        ////Serial.println(header.from_node, OCT);
        
        if (solicitud==0) {  // Se hemos recibido un 0 la luz está apagada.
          Estado=false;
        } else if (solicitud==1) { // Un 1 luz encendida.
          Estado=true;
        }
  
      } 
    }
  
}
  
boolean Luz::getEstado() {return Estado;}

/////////////////////////////////////////////////////////////////////////////////////////////////

const byte maxLuces=9;
Luz luces[maxLuces]; // Tenemos 9 luces (entrada, salon, cocina, dormitorio 1, dormitorio 2, dormitorio 3, dormitorio 4, aseo, garaje) - faltaría añadir pasillo y 2ª del salon (ambas controladas por el nodo 01)


/////////////////////////////////////////////////////////////////////////////////////////////////
// Clase Temperatura - hereda de la clase módulo
/////////////////////////////////////////////////////////////////////////////////////////////////

class Temperatura : public Modulo {
  private :
    float Valor;

  public:
    void actualizar();
    void setValor(float t);
    float getValor();
};
void Temperatura::setValor(float t) {Valor=t;}
void Temperatura::actualizar() {

  // Actualizamos el estado de la red antes de comunicar.
  network.update();

  ////Serial.print("Pidiendo actualizacion de temperatura al nodo : ");
  ////Serial.println(NumeroNodo, OCT);
  
  unsigned int solicitud = VER_TEMPERATURA; // Solicitar actualización datos.

  RF24NetworkHeader header(NumeroNodo, 't');
  if (!network.write(header, &solicitud, sizeof(solicitud))) {
    ////Serial.println(F("Send fail"));
      
    } else {
      ////Serial.print(F("Send OK: ")); //Serial.print(solicitud);  //Serial.print(" al nodo : "); //Serial.println(NumeroNodo, OCT);
  }

  delay(20); // Damos tiempo para que el nodo nos mande el paquete con los datos.
  network.update();
  
    while(network.available()){
    RF24NetworkHeader header;
    network.read(header, &Valor, sizeof(float));
    if (header.from_node==NumeroNodo) {
      
        
        ////Serial.print("Recibimos : ");
        ////Serial.print(Valor);
        ////Serial.print(" del nodo :");
        ////Serial.println(header.from_node, OCT);
      
      }
    }
}
float Temperatura::getValor() {return Valor;}
/////////////////////////////////////////////////////////

const byte maxTemperaturas=2;
Temperatura temperaturas[maxTemperaturas]; // Tenemos 2 medidas de temperatura (salon y dormitorio 3)

// Definimos un tipo de dato enumerativo para la posición de la persiana.
enum PosicionPersiana {bajada=0, un_cuarto, mitad, tres_cuartos, subida};

/////////////////////////////////////////////////////////////////////////////////////////////////
// Clase Persiana - hereda de la clase módulo
/////////////////////////////////////////////////////////////////////////////////////////////////
class Persiana : public Modulo {
  private :

    PosicionPersiana Estado;

  public:
    
    void setEstado(PosicionPersiana estado);
    void actualizar(); 
    PosicionPersiana getEstado();
};

void Persiana::setEstado(PosicionPersiana estado) 
  {
    if (estado!=Estado){
      // Introducir el codigo para modificar las persianas través del nodo RF24 correspondiente.
      Estado=estado;
    }
    
  }
    
void Persiana::actualizar() {
  
  // Introducir el codigo para obtener el estado las persianas a través del nodo RF24 correspondiente. 
  // Estado = ....
}

PosicionPersiana Persiana::getEstado() {return Estado;}

/////////////////////////////////////////////////////////

const byte maxPersianas=3;
Persiana persianas[maxPersianas]; // Tenemos 3 persianas con control domótico (salon, dormitorio 1 y dormitorio 3)

// Definimos un tipo de dato enumerativo para el estado del AACC.
enum EstadoAACC {apagado=0, encendido, automatico, cool, dry, fan, heat, quiet};


/////////////////////////////////////////////////////////////////////////////////////////////////
// Clase mAACC - hereda de la clase módulo
/////////////////////////////////////////////////////////////////////////////////////////////////
class mAACC : public Modulo {
  private :
    EstadoAACC Estado;
    byte Fan;
    float Temperatura;

  public:
    
    void setEstado(EstadoAACC estado);
    void setTemperatura(float temperatura);
    void setFan(byte fan);
    void actualizar(); 
    EstadoAACC getEstado();
    float getTemperatura();
    byte getFan();
};

void mAACC::setEstado(EstadoAACC estado) {
    if (estado!=Estado){
      // Introducir el codigo para establecer el funcionamiento del AACC a través del nodo RF24 correspondiente.
      Estado=estado;
    }
    
}

void mAACC::setTemperatura(float temperatura ){
  
}

void mAACC::setFan(byte fan) {Fan=fan;}
    
void mAACC::actualizar() {
  
  // Introducir el codigo para obtener el estado del AACC a través del nodo RF24 correspondiente. 
  // Estado = ....
}
EstadoAACC mAACC::getEstado() {return Estado;}
float mAACC::getTemperatura() {return Temperatura;}
byte mAACC::getFan() {return Fan;}

/////////////////////////////////////////////////////////

mAACC aire;

/////////////////////////////////////////////////////////////////////////////////////////////////
// Clase Puerta - hereda de la clase módulo
/////////////////////////////////////////////////////////////////////////////////////////////////
class Puerta : public Modulo {
  private :
    boolean Estado; // false = cerrada

  public:
    void actualizar(); 
    boolean getEstado();
};


    

void Puerta::actualizar() {
  
  network.update();
  
  ////Serial.print("Pidiendo estado puerta al nodo : ");
  ////Serial.println(NumeroNodo, OCT);
  
  unsigned int solicitud = ESTADO_PUERTA; // Solicitar actualización datos.
  //boolean enviado=false;

  RF24NetworkHeader header(NumeroNodo, 't');
  if (!network.write(header, &solicitud, sizeof(solicitud))) {
    ////Serial.println(F("Send fail"));
      
    } else {
      ////Serial.print(F("Send OK: ")); //Serial.print(solicitud);  //Serial.print(" al nodo : "); //Serial.println(NumeroNodo, OCT);
  }

  delay(20);
  network.update();
  
    while(network.available()){
    RF24NetworkHeader header;
    network.read(header, &solicitud, sizeof(unsigned long));
    if (header.from_node==NumeroNodo) {
      
        
        ////Serial.print("Recibimos : ");
        ////Serial.print(solicitud);
        ////Serial.print(" del nodo :");
        ////Serial.println(header.from_node, OCT);
        if (solicitud==0) {
          Estado=true;
        } else if (solicitud==1) {
          Estado=false;
        }
  
      }
    }
}

boolean Puerta::getEstado() {return Estado;}

/////////////////////////////////////////////////////////

const byte maxPuertas=2;
Puerta puertas[maxPuertas]; // Tenemos información sobre 2 puertas (entrada y garaje)

/////////////////////////////////////////////////////////////////////////////////////////////////
// Clase Consumo - hereda de la clase módulo
/////////////////////////////////////////////////////////////////////////////////////////////////
class Consumo : public Modulo {
  private :
  float Valor;
 

  public:
    void actualizar(); 
    float getValor();
};

void Consumo::actualizar() {
  
  // Introducir el codigo para obtener el estado de la luz a través del nodo RF24 correspondiente. 
  // Consumo = ....

  network.update();

  //Serial.print("Pidiendo actualizacion de PAGINA_CONSUMO al nodo : ");
  //Serial.println(NumeroNodo, OCT);
  
  unsigned int solicitud = VER_CONSUMO; // Solicitar actualización datos.
  //boolean enviado=false;

  RF24NetworkHeader header(NumeroNodo, 't');
  if (!network.write(header, &solicitud, sizeof(solicitud))) {
    //Serial.println(F("Send fail"));
      
    } else {
      //Serial.print(F("Send OK: ")); //Serial.print(solicitud);  //Serial.print(" al nodo : "); //Serial.println(NumeroNodo, OCT);
  }

  delay(70);
  network.update();
  
    while(network.available()){
    RF24NetworkHeader header;
    network.read(header, &Valor, sizeof(float));
    if (header.from_node==NumeroNodo) {
      
        if (Valor <2) {Valor=0;}
        ////Serial.print("Recibimos : ");
        ////Serial.print(Valor);
        ////Serial.print(" del nodo :");
        ////Serial.println(header.from_node, OCT);
      
      }
    }
    
}

float Consumo::getValor() {return Valor;}

/////////////////////////////////////////////////////////

const byte maxConsumos=2;
Consumo consumos[maxConsumos]; // Tenemos información sobre 2 consumos (entrada (consumo general de la vivienda) y cocina)

/////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////
// Configuración inicial de Arduino /////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////

void setup() {
  // Creamos los elementos domoticos
  // Luces

  luces[0].setEstancia("Entrada");
  luces[0].setNumeroNodo(03);

  luces[1].setEstancia("Salon");
  luces[1].setNumeroNodo(01);

  luces[2].setEstancia("Cocina");
  luces[2].setNumeroNodo(04);

  luces[3].setEstancia("Dormitorio 1");
  luces[3].setNumeroNodo(011);
  
  luces[4].setEstancia("Dormitorio 2");
  luces[4].setNumeroNodo(021);

  luces[5].setEstancia("Dormitorio 3");
  luces[5].setNumeroNodo(031);

  luces[6].setEstancia("Dormitorio 4");
  luces[6].setNumeroNodo(014);

  luces[7].setEstancia("Aseo");
  luces[7].setNumeroNodo(051);

  luces[8].setEstancia("Garaje");
  luces[8].setNumeroNodo(013);

  // Temperaturas

  temperaturas[0].setEstancia("Salon");
  temperaturas[0].setNumeroNodo(00);

  temperaturas[1].setEstancia("Dormitorio 3");
  temperaturas[1].setNumeroNodo(031);

  // Persianas

  persianas[0].setEstancia("Salon");
  persianas[0].setNumeroNodo(00);

  persianas[1].setEstancia("Dormitorio 1");
  persianas[1].setNumeroNodo(011);

  persianas[2].setEstancia("Dormitorio 3");
  persianas[2].setNumeroNodo(031);

  // AACC

  aire.setEstancia("Salon");
  aire.setNumeroNodo(02);

  // Puertas

  puertas[0].setEstancia("Entrada");
  puertas[0].setNumeroNodo(03);

  puertas[1].setEstancia("Garaje");
  puertas[1].setNumeroNodo(013);

  // Consumos

  consumos[0].setEstancia("Entrada");
  consumos[0].setNumeroNodo(03);

  consumos[1].setEstancia("Cocina");
  consumos[1].setNumeroNodo(04);
  
  Serial.begin(115200);
  // Iniciamos el módulo de radio RF
  radio.begin();
  radio.setPALevel(RF24_PA_HIGH); // Potencia alta (no máxima. Algo menos de consumo)
  network.begin(/*Canal RF de la banda de 2.4 GHz*/ 90, /*dirección que le asiganamos a este nodo*/ this_node);
  //radio.setRetries(5,5);
  //radio.setAutoAck(1);
  
  // En caso de configuración manual de la red. (Una configuración automática mediante DHCP no nos permitirá conocer la IP a la que acceder por el navegador).
  Ethernet.begin(mac, ip); //, gateway, subnet); // Si optamos por DHCP deberíamos ejecutar "Ethernet.begin(mac);". (Con gateway y subnet no funciona la peticio de IP púlbica)
  server.begin(); // Inciamos el servidor
  
  while (!Serial) { // Esperamos a que haya conxión serie para debugging
    ;
  }
  
  ///Serial.print(F("Server started at "));
  ////Serial.println(Ethernet.localIP());
  

  ////////////////////////////////////
  //Comprobar si está pulsado el pulsador de resteo de contraseña (de inicio user=admin;pass=1234)
  // Si está pulsado (PIN DIGITAL 2 COMO INPUT = HIGH) ejecutamos función de reseteo de user y pass
  ////////////////////////////////////
  
  pinMode(RESETPIN, INPUT);
  if (digitalRead(RESETPIN)) resetearUsuario();

  ///////////////////////////////////////////////////////////////////////////////////////////////////
  // Leer datos de usuario y contraseña en EEPROM
  ///////////////////////////////////////////////////////////////////////////////////////////////////
  leerDatosUsuarioEeprom();

  // Iniciamos el contador del tiempo para controlar cada cuanto debemos actualizar la IP en el DDNS.
  tiempoDDNS = millis(); 
  
  //obtenerIPpublica(ipPublica);
    
    Serial.println("La IP publica de nuestro host es: ");
    for (int i=0; i<4;i++) {
      if(i>0) Serial.print(".");
      Serial.print(ipPublica[i]);
    }
  Serial.println();
  
  actualizaIPDNS();
  tiempoDDNS=millis();
  
}

/////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////
// Funciones que utilizaremos en el programa ////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////////////////////
// Función para actualizar nuestra IP en el servidor DNS
/////////////////////////////////////////////////////////////////////////////////////////////////
void actualizaIPDNS() {

  // Datos cuenta en NO-IP
  String userpassword="########################################"; // En base64 user:password de la cuenta NO-IP
  
  EthernetClient client1;
   
  byte ipPublicaAct[4];
  memcpy (ipPublicaAct,ipPublica,4);
  
  Serial.println("La IP publica de nuestro host anteriormente era: ");
    for (int i=0; i<4;i++) {
      if(i>0) Serial.print(".");
      Serial.print(ipPublicaAct[i]);
    }
  
  Serial.println();
  
  obtenerIPpublica(ipPublica);
  
  
  
  Serial.println("La IP publica de nuestro host actualmente es: ");
  for (int i=0; i<4;i++) {
    if(i>0) Serial.print(".");
    Serial.print(ipPublica[i]);
  }
 
  if ((memcmp(ipPublicaAct,ipPublica,4))) { // memcmp compara los dos arrays y si son iguales devuelve un 0.
    
    Serial.println("Actualizando IP en DDNS");
    
    delay(100);
    
    // ver: https://www.noip.com/integrate/request
    
    String actualizar = "GET /nic/update?hostname=sergiodomos.ddns.net&myip=";   
    //int j=0;
    //char temp[4];
    for (int i=0;i<4;i++) {
      String anadir=String(ipPublica[i]);
      //Serial.println("String de la IP : ");
      //Serial.println(anadir);
      
      //anadir.toCharArray(temp,4);
      //strcat(actualizar, temp);
      actualizar += anadir;
      if (i<3) actualizar +=".";
  
    }
    actualizar += " HTTP/1.0";
    
    Serial.println();
    Serial.print("La peticion a realizar es : ");
    Serial.println(actualizar);
    //char temp[115];
    //actualizar.toCharArray(temp,actualizar.length()+1);
    
    
    char server1[] = "dynupdate.no-ip.com"; // No añadir HTTP:// al principio o no funciona
    Serial.print("Direccion del servidor : ");
    Serial.println(server1);
    
    //delay(20);
    
    if (client1.connect(server1,80)) {
        Serial.print("Client is at ");
        Serial.println(Ethernet.localIP());
        Serial.print("Conectado a: ");
        Serial.println(server1);
        
        //delay(200); // Esperar respuesta del servidor
        uint32_t tiempo=millis();
        while ((millis()-tiempo <10000) && (!client1)) {
          if ( ((millis()-tiempo) % 1) == 0) Serial.println(".");        
        }
        
        if (millis()-tiempo>10000) {
          Serial.println("No se ha recibido respuesta en 10 segundos");
        } 
        else {
          client1.println(actualizar);
          client1.println("Host: dynupdate.no-ip.com");
          
          client1.print("Authorization: Basic ");
          //client1.print(username);
          //client1.print(F(":"));
          client1.println(userpassword);
          
          client1.println("User-Agent: Sergio_Arduino_Client/0.0 mantain serferfe80@gmail.com");
          client1.println();
          
          Serial.println(actualizar);
          Serial.println("Host: dynupdate.no-ip.com");
          
          Serial.print("Authorization: Basic ");
          //Serial.print(username);
          //Serial.print(F(":"));
          Serial.println(userpassword);
         
          Serial.println("User-Agent: Sergio_Arduino_Client/0.0 mantain serferfe80@gmail.com");
          Serial.println();

          delay(20);

          uint32_t tiempo=millis();
          while ((millis()-tiempo <10000) && (!client1.connected())) {
            if ( ((millis()-tiempo) % 1) == 0) Serial.println(".");        
          }

          
          if (millis()-tiempo>10000) {
            Serial.println("No se ha recibido respuesta en 10 segundos");
          } else {
            Serial.print("Recibimos : ");
          }
          
          while(client1.connected()) {
                        
            while (client1.available()) {
              char c;
              c = client1.read();//Leemos la petición HTTP carácter por carácter
              Serial.print(c);//Visualizamos la petición HTTP por el Monitor Serial
        //cadena.concat(c);//Unimos el String 'cadena' con la petición HTTP (c). De esta manera convertimos la petición HTTP a un String

         //Ya que hemos convertido la petición HTTP a una cadena de caracteres, ahora podremos buscar partes del texto.
         //posicion=cadena.indexOf("<!--IPADDR="); //Guardamos la posición de la instancia "LED=" a la variable 'posicion'

       
            }
          }
          
        }
                
      } 
      
      else {
        Serial.println("No concetado. No se ha podido hacer la actualización");
        return;
      }
    
    //Serial.println("Enviada peticion de actualización");
    //Serial.println(temp);
    
    
      client1.stop();
    } 
    else {
      Serial.println("\nNo se actualiza");
    }
  Serial.println("YA");  
}

/////////////////////////////////////////////////////////////////////////////////////////////////
// Función para obtener nuestra IP publica
/////////////////////////////////////////////////////////////////////////////////////////////////
void obtenerIPpublica(byte ip[]) {
  
  EthernetClient client1;
  char server1[] = "ip.changeip.com";
  delay(100);
  
  
  if(client1.connect(server1,80)){
    Serial.print("Client is at ");
    Serial.println(Ethernet.localIP());
    Serial.print("Conectado a: ");
    Serial.println(server1);
    
  } else {
    Serial.println("No concetado.");
    return;
  }

  if (client1) {
  
    client1.println("GET / HTTP/1.1");
    client1.println("Host: ip.changeip.com");
    client1.println("Connection: close");
    client1.println();

    String cadena=""; //Creamos una cadena de caracteres vacía
    String bytes=""; //Creamos una cadena de caracteres vacía
    int i=0;
    int j=0;
    int z=0;
    int posicion;
    char c;
    //int ippublica[4];


    
    while (client1.connected()) {
      if (client1.available()) {
        
        c = client1.read();//Leemos la petición HTTP carácter por carácter
        //Serial.write(c);//Visualizamos la petición HTTP por el Monitor Serial
        cadena.concat(c);//Unimos el String 'cadena' con la petición HTTP (c). De esta manera convertimos la petición HTTP a un String

         //Ya que hemos convertido la petición HTTP a una cadena de caracteres, ahora podremos buscar partes del texto.
         posicion=cadena.indexOf("<!--IPADDR="); //Guardamos la posición en la variable 'posicion'
      
      }
    }  
    
   
    //delay(1);
    client1.stop();// Cierra la conexión
    
    do {
       if(cadena.charAt(posicion+11+i)=='.' or cadena.charAt(posicion+11+i)=='-'){
        bytes=cadena.substring(posicion+11+j,posicion+11+i);
        ip[z]=bytes.toInt();
        i++;
        j=i;   
        z++;
       }
       i++;
     
    } while (cadena.charAt(posicion+10+i)!='-');
    /*
    Serial.println("La IP publica de nuestro host es: ");
    for (int i=0; i<4;i++) {
      if(i>0) Serial.print(".");
      Serial.print(ip[i]);
    }*/
    
  }
  client1.stop();// Cierra la conexión
}


/////////////////////////////////////////////////////////////////////////////////////////////////
// Función para resetear los valores de usuario y contraseña iniciales
/////////////////////////////////////////////////////////////////////////////////////////////////
void resetearUsuario() {
  ////Serial.println("Reseteando usuario.....");
  almacenarDatosUsuarioEeprom("admin","1234");
}

/////////////////////////////////////////////////////////////////////////////////////////////////
// Función para leer los datos usuario y contrasñea almacenados en la EEPROM
/////////////////////////////////////////////////////////////////////////////////////////////////
void leerDatosUsuarioEeprom() {
  ////Serial.println("Leyendo datos de EEPROM");
  char valorLeido;
  int addr=0;
  
  USER="";
  PASS="";
  
  // Datos de usuario
  ////Serial.println("Leyendo datos de usuario");
  for (int i=0;i<15;i++){
    if (EEPROM.read(i)) {
      valorLeido=EEPROM.read(i);
      USER += valorLeido;
    }
  }
  ////Serial.println(USER);

  // Datos de password
  ////Serial.println("Leyendo datos de password");
  for (int i=0;i<15;i++){
    if (EEPROM.read(i+15)) {
      valorLeido=EEPROM.read(i+15);
      PASS += valorLeido;
    }
  }
  ////Serial.println(PASS);

  pinMode(3, OUTPUT);

  if (USER == "admin" && PASS== "1234") digitalWrite(3, HIGH);
  else digitalWrite(3, LOW);
  
  
}

/////////////////////////////////////////////////////////////////////////////////////////////////
// Función para guardar datos usuario y contrasñea en la EEPROM
/////////////////////////////////////////////////////////////////////////////////////////////////
void almacenarDatosUsuarioEeprom(String user, String pass) {
  
  // Almacenamos datos de usuario hasta 15 carácteres (del 0 al 14)
  // La dirección 15 corresponderá al inicio de la contraseña 
  int addr = 0;
  if (user.length()>14) {
    ////Serial.println("Error en la longitud del string de usuario");
    return;
  }
  ////Serial.print("Almacenando usuario : ");
  for (int i=0; i<15;i++) {
    if (i<=user.length()){
      EEPROM.write(addr, user.charAt(i));
      ////Serial.print(user.charAt(i));
      delay(50);
    } else {
      EEPROM.write(addr, 0);  // Colocamos ceros (byte, NO caracter 0) después del usuario hasta completar los 15 bytes
      delay(50);
    }
    
    addr++;
    
  }
  ////Serial.print("\nDesde la dirección : 0 a la ");
  ////Serial.println(user.length());



  if (pass.length()>14) {
    ////Serial.println("Error en la longitud del string del password");
    return;
  }
  
  ////Serial.print("Almacenando pass : ");
  for (int i=0; i<15;i++) {
    if (i<=pass.length()){
      EEPROM.write(addr, pass.charAt(i));
      ////Serial.print(pass.charAt(i));
      delay(50);
    } else {
      EEPROM.write(addr, 0);  // Colocamos ceros (byte, NO caracter 0) después de la contraseña hasta completar los 15 bytes
      delay(50);
    }
    addr++;
    
  }

  ////Serial.print("\nDesde la dirección : 15 a la ");
  ////Serial.println(pass.length()+15);
 
 
}


/////////////////////////////////////////////////////////////////////////////////////////////////
// Función para actualizar el tiempo de inactividad
/////////////////////////////////////////////////////////////////////////////////////////////////
uint32_t actualizarInactividad() {
  return (millis()-momentoUltimaActividad);
}
/////////////////////////////////////////////////////////////////////////////////////////////////
// Función para comprobar que estamos dentro del tiempo permitido de inactividad.
/////////////////////////////////////////////////////////////////////////////////////////////////
bool comprobarTiempoInactivo() {
  tiempoInactivo=millis()-momentoUltimaActividad;
  ////Serial.print("Tiempo inactivo (ms) : ");
  ////Serial.println(tiempoInactivo);
  return(tiempoInactivo < MAX_TIEMPO_INACTIVO);
}
/////////////////////////////////////////////////////////////////////////////////////////////////
// Función para actualizar el momento en el que se realiza la ultima actividad permitida.
/////////////////////////////////////////////////////////////////////////////////////////////////
void actualizarTiempoActividadAutorizada(){
  momentoUltimaActividad=millis();
}

/////////////////////////////////////////////////////////////////////////////////////////////////
// Función que nos devuelve el valor de la temperatura obtenida del sensor LM35 en este microcontrolador
/////////////////////////////////////////////////////////////////////////////////////////////////
float leerTemperatura() {
  /**** Dado que el LM35DZ tiene un rango de valores desde -55ºC (correspndiente a -550mV) hasta 150ºC (correspondiente a 1500mV), y la temperatura ambiente puede rondar desde los -10ºC hasta los 45ºC (por ejemplo) tendremos valores entre -10mV y 450 mV.
   *  Por lo tanto podemos modificar el valor de referencia de la tensión analógica del microcontrolador para que la precisión sea mayor. 
   */
  analogReference(INTERNAL1V1);
  uint32_t valorLM35=0;
  for (int i=0;i<10000;i++) {
    valorLM35 += analogRead(PIN_LM35); // Leemos en el pin analógico al que está conectado el LM35 10000 veces para sacar un promedio
  }
  valorLM35 = valorLM35/10000; // promedio
  float milivoltios = (1100 /1023) * valorLM35; // Tensión en milivoltios. 1100 son los milivoltios de la tensión de referencia. 1023 es la resolución del conversor AD (10 bits ->1024 valores, del 0 al 1023)
  float temp=milivoltios/10;
  ////Serial.print("Temperatura local : ");
    
  ////Serial.println(temp);
  return temp;
}

/////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////
// Funciones para calcular las claves pública (B_1), privada (b) y compartida (k) (Diffie-Hellman)
// Extraido de https://github.com/DevMomo/Diffie-Hellman-Arduino con modificaciones para adaptarlo a este proyecto
/////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////// 
/*  CMPUT 296/114 - Assignment 1 Part 2 - Due 2012-10-03
 
 Version 1.2 2012-10-01
 
 By: Monir Imamverdi
 Michael Nicholson
 */
/////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////////////////////
// Función para generar la clave privada 'b'
/////////////////////////////////////////////////////////////////////////////////////////////////

uint32_t keyGen(){
  randomSeed(analogRead(2));
  
  return random(1,PRIMO); // Valor aleatorio entre 1 y el número PRIMO (p) del algoritmo Diffie-Hellman
}

/////////////////////////////////////////////////////////////////////////////////////////////////
// Función que devuele el resto de una operación ((a*b) mod m) sin perder dígitos por espacio de memoria de las variables.
/////////////////////////////////////////////////////////////////////////////////////////////////
uint32_t mul_mod(uint32_t a, uint32_t b, uint32_t m){


  uint32_t result = 0; //variable to store the result
  uint32_t runningCount = b % m; //holds the value of b*2^i

  for(int i = 0 ; i < 32 ; i++){

    if(i > 0) runningCount = (runningCount << 1) % m;
    if(bitRead(a,i)){
      result = (result%m + runningCount%m) % m; 

    } 

  }
  return result;
}

/////////////////////////////////////////////////////////////////////////////////////////////////
// Función que devuelve el resto de la operación ((b^e) mod m) sin perder dígitos por espacio de memoria de las variables.
/////////////////////////////////////////////////////////////////////////////////////////////////
//The pow_mod function to compute (b^e) % m that was given in the class files  
uint32_t pow_mod(uint32_t b, uint32_t e, uint32_t m)
{
  uint32_t r;  // result of this function

  uint32_t pow;
  uint32_t e_i = e;
  // current bit position being processed of e, not used except for debugging
  uint8_t i;

  // if b = 0 or m = 0 then result is always 0
  if ( b == 0 || m == 0 ) { 
    return 0; 
  }

  // if e = 0 then result is 1
  if ( e == 0 ) { 
    return 1; 
  }

  // reduce b mod m 
  b = b % m;

  // initialize pow, it satisfies
  //    pow = (b ^ (2 ^ i)) % m
  pow = b;

  r = 1;

  // stop the moment no bits left in e to be processed
  while ( e_i ) {
    // At this point pow = (b ^ (2 ^ i)) % m

    // and we need to ensure that  r = (b ^ e_[i..0] ) % m
    // is the current bit of e set?
    if ( e_i & 1 ) {
      // this will overflow if numbits(b) + numbits(pow) > 32
      r= mul_mod(r,pow,m);//(r * pow) % m; 
    }

    // now square and move to next bit of e
    // this will overflow if 2 * numbits(pow) > 32
    pow = mul_mod(pow,pow,m);//(pow * pow) % m;

    e_i = e_i >> 1;
    i++;
  }

  // at this point r = (b ^ e) % m, provided no overflow occurred
  return r;
}

/////////////////////////////////////////////////////////////////////////////////////////////////
// Función que realiza la autenticación del usuario (Se utiliza Diffie-Hellman para el intercambio
// de claves privadas compartidas y AES_128 simple para el cifrado del usuario y contraseña.
// Esta función obtiene la clave pública compartida A del navegador, y mediante Diffie-Hellman
// obtiene la clave secreta k. 
// Lee los datos de la cabera HTTP (datos cifrados con AES) junto con el valor de A. Los separa,
// los descifra y los compara con el nombre de usuario y contraseña permitido.
/////////////////////////////////////////////////////////////////////////////////////////////////
boolean comprobarAutentificacion(EthernetClient client) {
 
  String usuarioHTTP = (lectura.substring(lectura.indexOf("?user=")+6,lectura.indexOf("&pass")));
  String passwordHTTP = (lectura.substring(lectura.indexOf("&pass=")+6,lectura.indexOf("&A1")));
  String A1 = (lectura.substring(lectura.indexOf("&A1=")+4,lectura.indexOf(" HTTP")));
  uint32_t A=A1.toInt();
 
  uint32_t k_uint32 = pow_mod(A, b, PRIMO);

  ////Serial.println(k_uint32);

  String usuario=desencriptar_aes_128_HTTP(usuarioHTTP, k_uint32);
  ////Serial.println(usuario);
  //delay(100);
  String password=desencriptar_aes_128_HTTP(passwordHTTP, k_uint32);
  //Serial.println(password);

  if (usuario==USER && password==PASS) {
    return true;
    //actualizarTiempoActividadAutorizada();
  } else {
    return false;
  }
    
}

/////////////////////////////////////////////////////////////////////////////////////////////////
// Función que realiza el cambio de los datos de acceso descifrando los datos primero
////////////////////////////////////////////////////////////////////////////////////////////////

boolean cambioDatosAcceso(EthernetClient client) {
 
  String usuarioHTTP = (lectura.substring(lectura.indexOf("?newuser=")+9,lectura.indexOf("&newpass")));
  String passwordHTTP = (lectura.substring(lectura.indexOf("&newpass=")+9,lectura.indexOf("&A1")));
  String A1 = (lectura.substring(lectura.indexOf("&A1=")+4,lectura.indexOf(" HTTP")));
  uint32_t A=A1.toInt();
 
  uint32_t k_uint32 = pow_mod(A, b, PRIMO);

  //Serial.println(k_uint32);

  String usuario=desencriptar_aes_128_HTTP(usuarioHTTP, k_uint32);
  //Serial.println(usuario);
  //delay(100);
  String password=desencriptar_aes_128_HTTP(passwordHTTP, k_uint32);
  //Serial.println(password);

  almacenarDatosUsuarioEeprom(usuario, password);

  leerDatosUsuarioEeprom();
  
  return true;
    
}

/////////////////////////////////////////////////////////////////////////////////////////////////
// Función auxiliar para obtener el texto en claro a partir de un texto cifrado HTTP y la clave de cifrado
// AES 128 simple.
/////////////////////////////////////////////////////////////////////////////////////////////////
String desencriptar_aes_128_HTTP(String entradaHTTP, uint32_t k) {
  
  String str="";
  String salidaString="";
  
  // Convertimos la clave privada k en string para repetirla hasta ocupar 16 bytes para trabajar con la librería AESLib.h (aes128_dec_single(uint8_t [16] key, char[16] data)
  str=String(k);
 
  for (int i=0;16-str.length();i++){
    str=str+str.charAt(i);
  }
 
  //Pasamos el string key a un array de bytes (uint8_t) para trabajar con la función aes128_dec_single() de la librería AESLib.h
  uint8_t key[str.length()+1];
  str.getBytes(key,str.length()+1);
  
  char salida[str.length()]; // La función aes128_dec_single() trabaja con array de carácteres (bytes) por lo que debemos pasar el string a array de char
   
  //Obtenemos los bytes de la lectura HTTP eliminando loas cadenas "%2C" (que representan comas)
  int i=0;
  while (entradaHTTP.length()>0 && i<16){
    String salidaString=entradaHTTP.substring(0,lectura.indexOf("%2C"));
    entradaHTTP.remove(0,entradaHTTP.indexOf("%2C")+3);
  
    salida[i]=(char)(salidaString.toInt());
    i++;
  }

  aes128_dec_single(key, salida); // Función que transforma los datos de la variable "salida", cifrados, a datos descifrados.

  // Pasamos el array de chars con el usuario desencriptado a un string y eliminamos lo carácteres de relleno "~" que se han añadido por el navegador.
  for (int i=0; i<sizeof(salida) ;i++) {
    if (salida[i]!='~')  salidaString += salida[i];
  }
  
  return salidaString;
}


/////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////
// Funciones para el manejo de las peticiones HTTP y los módulo de RF
/////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////// 

/////////////////////////////////////////////////////////////////////////////////////////////////
// Función que comprueba si hay alguna petición (cliente) y devuelve los datos leídos de la primera línea HTTP
/////////////////////////////////////////////////////////////////////////////////////////////////

String checkForClient(EthernetClient client){
  lectura="";
  if (client) { // si existe un cliente


  // Una petición http acaba con una línea en vacía
  boolean actualLineaVacia = true;
  boolean primeraLineaLeida = false;
    //boolean sentHeader = false;

    while (client.connected()) { 
      if (client.available()) {
        char c = client.read(); // Leemos los carácteres de entrada HTTP.
        Serial.write(c); // Línea debug. Mostramos la lectura por el monitor serie.

        if (c == '\n' && actualLineaVacia) { // Hemos acabdo de recibir la petición HTTP del cliente
          respuestaHTTP(client);
          break;
        }
        if (c == '\n') {
          // you're starting a new line
          actualLineaVacia = true;
          primeraLineaLeida = true;
        } else if (c != '\r') {
          // you've gotten a character on the current line
          actualLineaVacia = false;
        }

        if (!primeraLineaLeida){
          lectura.concat(c); // Vamos montando la cadena de lectura con la petición HTTP. Solo guardamos la primera línea.
        }
      }
    }
  //Serial.print(F("Esto se ha guardado en lectura: "));
  //Serial.println(lectura);
  } 

  return lectura;  
  
}


/////////////////////////////////////////////////////////////////////////////////////////////////
// Función que envía una respuesta HTTP estandar (cabcera) al cliente.
/////////////////////////////////////////////////////////////////////////////////////////////////
void respuestaHTTP(EthernetClient client) {
  // Enviar una respuesta HTTP estandar
  client.println(F("HTTP/1.1 200 OK"));
  client.println(F("Content-Type: text/html"));
  client.println(F("Cache-Control: max-age=0, no-cache"));
  client.println(F("Connection: close")); 
  client.println(F("Server: Servidor Arduino Mega 2560"));
  client.println(); // Final de la respuesta HTTP. A partir de aqui vendrán los datos (páginas a mostrar o enviar al navegador).

}


/////////////////////////////////////////////////////////////////////////////////////////////////
// Función que recoje información de los módulos de RF correspondientes a un cierto elemento
/////////////////////////////////////////////////////////////////////////////////////////////////

void recogerInfo(byte elemento) {
  switch (elemento) {
    case LUCES :
    //Serial.println("Recogiendo datos de :" + elementos[elemento]);
    for (byte i=0;i<maxLuces;i++) {
      luces[i].actualizar();
    }
    break;

    case  TEMPERATURA :
    //Serial.println("Recogiendo datos de :" + elementos[elemento]);
   
    temperaturas[0].setValor(leerTemperatura());
    for (byte i=1;i<maxTemperaturas;i++) {
      temperaturas[i].actualizar();
    }
    break;

    case PERSIANAS :
    //Serial.println("Recogiendo datos de :" + elementos[elemento]);
    for (byte i=0;i<maxPersianas;i++) {
      persianas[i].actualizar();
    }
    break;   

    case AACC :
    //Serial.println("Recogiendo datos de :" + elementos[elemento]);
    aire.actualizar();
    break;

    case PUERTAS :
    //Serial.println("Recogiendo datos de :" + elementos[elemento]);
    for (byte i=0;i<maxPuertas;i++) {
      puertas[i].actualizar();
    }
    break;   
      
    case PAGINA_CONSUMO : 
    //Serial.println("Recogiendo datos de :" + elementos[elemento]);
    for (byte i=0;i<maxConsumos;i++) {
      consumos[i].actualizar();
    }
    break;   
  }
}

/////////////////////////////////////////////////////////////////////////////////////////////////
// Función que enciende la luz correspondiente
/////////////////////////////////////////////////////////////////////////////////////////////////
void encender(String luz) {
  //Serial.println("Encendiendo luz");
  for (byte i=0;i<numEstancias;i++){
    if (luz==luces[i].getEstancia()) {
      luces[i].setEstado(true);
    }
  }
}

/////////////////////////////////////////////////////////////////////////////////////////////////
// Función que apaga la luz correspondiente
/////////////////////////////////////////////////////////////////////////////////////////////////
void apagar(String luz) {
  //Serial.println("Apgando luz");  
  for (byte i=0;i<numEstancias;i++){
    if (luz==luces[i].getEstancia()) {
      luces[i].setEstado(false);
    }
  }
}

/////////////////////////////////////////////////////////////////////////////////////////////////
// Función que modifica la persiana correspondiente
/////////////////////////////////////////////////////////////////////////////////////////////////

void persiana(String persiana, PosicionPersiana posicion) {
  //Serial.println("Modificando persiana");
  for (byte i=0;i<numEstancias;i++){
    if (persiana==persianas[i].getEstancia()) {
      if (persianas[i].getEstado()!=posicion) {
        //Serial.println("Modificando persiana .....");
        persianas[i].setEstado(posicion); 
      }
      
    }
  }
}




/////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////
// Funciones para enviar páginas HTML al cliente (navegador)
/////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////////
// Función que ejecuta las funciones para mostrar la página de autorización errónea
/////////////////////////////////////////////////////////////////////////////////////////////////
void mensajeNoaut(EthernetClient client) {
  imprimirCabeceraHtml(0, AUTORIZAR, client);
  imprimirCSS(AUTORIZAR, client);
  impimirScriptCifrar(client);  
  imprimirCabeceraPagina(client);
  imprimirPagina(9, client);
  client.println(F("</html>"));
}

/////////////////////////////////////////////////////////////////////////////////////////////////
// Función que ejecuta las funciones para mostrar la página del menu principal
/////////////////////////////////////////////////////////////////////////////////////////////////
void menuFuncionesHtml(EthernetClient client) {
  imprimirCabeceraHtml(0, MENU, client);
  imprimirCSS(MENU, client);
  imprimirCabeceraPagina(client);
  imprimirPagina(MENU, client);
  client.println(F("</html>"));

  delay(1);
  
}


/////////////////////////////////////////////////////////////////////////////////////////////////
// Enviar página para pedir autorización.
/////////////////////////////////////////////////////////////////////////////////////////////////

void autentificar(EthernetClient client) {

  imprimirCabeceraHtml(0, AUTORIZAR, client);
  imprimirCSS(AUTORIZAR, client);
  impimirScriptCifrar(client);
  imprimirCabeceraPagina(client);
  imprimirPagina(AUTORIZAR, client);
  client.println(F("</html>"));
  
}

/////////////////////////////////////////////////////////////////////////////////////////////////
// Enviar página para pedir cambio de datos de acceso.
/////////////////////////////////////////////////////////////////////////////////////////////////

void paginaCambioDatosAcceso(EthernetClient client) {

  imprimirCabeceraHtml(0, CAMBIOPASSWORD, client);
  imprimirCSS(CAMBIOPASSWORD, client);
  impimirScriptCifrar(client);
  imprimirCabeceraPagina(client);
  imprimirPagina(CAMBIOPASSWORD, client);
  client.println(F("</html>"));
  
}


/////////////////////////////////////////////////////////////////////////////////////////////////
// Enviar código HTML. Cabcera (head) HTML (r = refresco de la página (si o no), 
// pagina = tipo de página (del las definidas con #define al incio), client = socket del cliente)
/////////////////////////////////////////////////////////////////////////////////////////////////
void imprimirCabeceraHtml(boolean r, byte pagina, EthernetClient client) {
  
  String url="/?elemento="+elementos[pagina];

  client.println(F("<!DOCTYPE HTML>"));
  client.println(F("<head>"));
  client.println(F("<title>TFG. UOC. Sistema domotico inalambrico con portal web.</title>"));
  client.println(F("<meta name=\"description\" content=\"TFG. UOC. Sistema domotico inalambrico con portal web.\"/>"));
  client.println(F("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"/>"));
  if (r) {
    client.print(F("<meta http-equiv=\"refresh\" content=\""));
    client.print(refreshPage);
    client.print(F("; url="));
    client.print(url);
    client.println(F("\">"));
    }
  client.println(F("<meta name=\"apple-mobile-web-app-capable\" content=\"yes\">"));
  client.println(F("<meta name=\"apple-mobile-web-app-status-bar-style\" content=\"default\">"));
  client.println(F("<meta name=\"viewport\" content=\"width=device-width, user-scalable=no\">"));
  client.println(F("<style type=\"text/css\">"));

}

/////////////////////////////////////////////////////////////////////////////////////////////////
// Enviar código HTML. Hoja de estilo CSS para la página
/////////////////////////////////////////////////////////////////////////////////////////////////
void imprimirCSS(int tipo, EthernetClient client) {

  client.println(F("html { height:100%; }"));
  client.println(F("  body {"));
  client.println(F("    height: 100%;"));
  client.println(F("    margin: 0;"));
  client.println(F("    font-family: arial, helvetica;"));
  //client.println("    background: #ffffff;"); // No hace falta indicar el color de fondo si es blanco.
  client.println(F("    -webkit-text-size-adjust: none;"));
  client.println(F("    background-image: url(\"http://www.uoc.edu/portal/system/modules/edu.uoc.presentations/resources/img/branding/logo-uoc-default.png\");"));
  client.println(F("  background-repeat: no-repeat;"));
  client.println(F("    background-size: 2cm;"));
  client.println(F("    background-position: left bottom;"));
  client.println(F(" }"));

  client.println(F("   .sergio {"));
  client.println(F("     color: #770000;"));
  client.println(F("     cursor: pointer;"));
  client.println(F("   }"));

  client.println(F(".view {"));
  client.println(F("    min-height: 100%;"));
  client.println(F("    overflow: auto;"));
  client.println(F("   }"));
  
  client.println(F(".header-wrapper { "));
  client.println(F("    height: 78px; "));
  client.println(F("    font-weight: bold; "));
  client.println(F("    text-shadow: rgba(0,0,0,0.7) -2px -2px 0; "));
  client.println(F("    border-top: solid 1px rgba(255,255,255,0.6); "));
  client.println(F("    border-bottom: solid 1px rgba(0,0,0,0.6); "));
  client.println(F("    color: #fff; "));
  client.println(F("    background-color: #8195af; "));
  client.println(F("    background-image: "));
  client.println(F("    -webkit-gradient(linear, left top, left bottom, "));
  client.println(F("    from(rgba(255,255,255,.4)), "));
  client.println(F("    to(rgba(255,255,255,.05)) ), "));
  client.println(F("    -webkit-gradient(linear, left top, left bottom, "));
  client.println(F("    from(transparent), "));
  client.println(F("    to(rgba(0,0,64,.9)) ); "));
  client.println(F("    background-repeat: no-repeat; "));
  client.println(F("    background-position: top left, bottom left; "));
  client.println(F("    -webkit-background-size: 100% 41px, 100% 42px; "));
  client.println(F("    -webkit-box-sizing: border-box; "));
  client.println(F("   }"));

  client.println(F(".header-wrapper h1 {"));
  client.println(F("   text-align: center;"));
  client.println(F("    font-size: 20px;"));
  client.println(F("    line-height: 35px;"));
  client.println(F("    margin: 0;"));
  client.println(F("   }"));

  client.println(F(".group-wrapper {"));
  client.println(F("    margin: 9px;"));
  client.println(F("    }"));

  client.println(F(".group-wrapper h2 {"));
  client.println(F("    text-align: center;"));
  client.println(F("    color: #4c566c;"));
  client.println(F("    font-size: 17px;"));
  client.println(F("    line-height: 0.8;"));
  client.println(F("    font-weight: bold;"));
  client.println(F("    text-shadow: #fff 0 1px 0;"));
  client.println(F("    margin: 20px 10px 12px;"));
  client.println(F("   }"));

  client.println(F(".group-wrapper h3 {"));
  client.println(F("    color: #4c566c;"));
  client.println(F("    font-size: 12px;"));
  client.println(F("    line-height: 1;"));
  client.println(F("    font-weight: bold;"));
  client.println(F("    text-shadow: #aaa 0 1px 0;"));
  client.println(F("    margin: 20px 10px 12px;"));
  client.println(F("   }"));

  client.println(F(".group-wrapper h4 {"));
  client.println(F("    color: #212121;"));
  client.println(F("    font-size: 14px;"));
  client.println(F("    line-height: 1;"));
  client.println(F("    font-weight: bold;"));
  client.println(F("    text-shadow: #aaa 1px 1px 3px;"));
  client.println(F("    margin: 5px 5px 5px;"));
  client.println(F("   }"));

  client.println(F(".group-wrapper table {"));
  client.println(F("    background-color: #fff;"));
  client.println(F("    -webkit-border-radius: 10px;"));
  client.println(F("    -moz-border-radius: 10px;"));
  client.println(F("    -khtml-border-radius: 10px;"));
  client.println(F("    border-radius: 10px;"));
  client.println(F("    font-size: 17px;"));
  client.println(F("    line-height: 20px;"));
  client.println(F("    margin: 9px 0 20px;"));
  client.println(F("    border: solid 1px #a9abae;"));
  client.println(F("    padding: 11px 5px 12px 5px;"));
  client.println(F("    margin-left:auto;"));
  client.println(F("    margin-right:auto;"));
  client.println(F("    -moz-transform :scale(1);"));
  client.println(F("    -moz-transform-origin: 0 0;"));
  client.println(F("   }"));
  
  switch (tipo) {
    case LUCES : // Luces. Tenemos que crear el tipo de estilo para la apariencia de los led's
    client.println(F(".green-circle {"));  // Círculo encendido.
    client.println(F("display: block;"));
    client.println(F("height: 23px;"));
    client.println(F("width: 23px;"));
    client.println(F("background-color: #ff0;"));
    client.println(F("-moz-border-radius: 11px;"));
    client.println(F("-webkit-border-radius: 11px;"));
    client.println(F("-khtml-border-radius: 11px;"));
    client.println(F("border-radius: 11px;"));
    client.println(F("margin-left: 1px;"));
    client.println(F("background-image: -webkit-gradient(linear, 0% 0%, 0% 90%, from(rgba(160, 160, 0, 0.8)), to(rgba(255, 255, 0, .9)));@"));
    client.println(F("border: 2px solid #ccc;"));
    client.println(F("-webkit-box-shadow: rgba(11, 140, 27, 0.5) 0px 10px 16px;"));
    client.println(F("-moz-box-shadow: rgba(11, 140, 27, 0.5) 0px 10px 16px; "));
    client.println(F("box-shadow: rgba(11, 140, 27, 0.5) 0px 10px 16px; "));
    client.println(F("}"));

    client.println(F(".black-circle {"));  // Círculo apagado.
    client.println(F("display: block;"));
    client.println(F("height: 23px;"));
    client.println(F("width: 23px;"));
    client.println(F("background-color: #990;"));
    client.println(F("-moz-border-radius: 11px;"));
    client.println(F("-webkit-border-radius: 11px;"));
    client.println(F("-khtml-border-radius: 11px;"));
    client.println(F("border-radius: 11px;"));
    client.println(F("margin-left: 1px;"));
    client.println(F("-webkit-box-shadow: rgba(11, 140, 27, 0.5) 0px 10px 16px;"));
    client.println(F("-moz-box-shadow: rgba(11, 140, 27, 0.5) 0px 10px 16px;"));
    client.println(F("box-shadow: rgba(11, 140, 27, 0.5) 0px 10px 16px;"));
    client.println(F("}"));

    client.println(F(".glare {")); // Brillo o destello
    client.println(F("position: relative;"));
    client.println(F("  top: 1;"));
    client.println(F("left: 5px;"));
    client.println(F("-webkit-border-radius: 10px;"));
    client.println(F("-moz-border-radius: 10px;"));
    client.println(F("-khtml-border-radius: 10px;"));
    client.println(F("border-radius: 10px;"));
    client.println(F("height: 1px;"));
    client.println(F("width: 13px;"));
    client.println(F("padding: 5px 0;"));
    client.println(F("background-color: rgba(200, 200, 200, 0.25);"));
    client.println(F("background-image: -webkit-gradient(linear, 0% 0%, 0% 95%, from(rgba(255, 255, 255, 0.7)), to(rgba(255, 255, 255, 0)));"));
    client.println(F("}"));

    break;
    }
  client.println(F("</style>"));
  client.println(F("</head>"));
   
}

/////////////////////////////////////////////////////////////////////////////////////////////////
// Enviar código HTML. Parte de JS para cifrar los datos usuario y contraseña en el navegador
// y que sean enviados por formulario en la petición GET al servidor (cifrados).
/////////////////////////////////////////////////////////////////////////////////////////////////
void impimirScriptCifrar(EthernetClient client) {

  // Primero calculamos los valores Diffie-Hellman
  //uint32_t b;
  b=keyGen();
  //b=123456;
  uint32_t B_1 = pow_mod(GENERADOR, b, PRIMO);

  // Empezamos a enviar el JS al navegador
  client.println(F("<script type=\"text/javascript\" src=\"http://peterolson.github.com/BigInteger.js/BigInteger.min.js\"></script>")); // Cargamos el JS para trabajar con números grandes en las operaciones del Diffie.
  client.println(F("<script type=\"text/javascript\" src=\"https://cdn.rawgit.com/ricmoo/aes-js/e27b99df/index.js\"></script>"));  // Cargamos el JS para trabajar con AES.
  client.println(F("<script type=\"text/javascript\">"));
  
  client.println(F("function diffie() {"));
  client.print(F("  var p="));
  client.print(String(PRIMO));
  client.println(F(";"));
  client.print(F("  var g="));
  client.print(String(GENERADOR));
  client.println(F(";"));
  client.println(F("  var a=Math.ceil(Math.random()*p);"));

  //client.println(F("  var a=9876543;"));
  
  client.println(F("  var A1=bigInt(g).modPow(a,p);"));
  
  client.print(F("  var B1=bigInt(")); // Obtener B1 del servidor WEB
  client.print(String(B_1));
  client.println(F(");"));
  client.println(F("var input_user = document.getElementById(1);"));
  client.println(F("var input_pass = document.getElementById(2);"));
  client.println(F("document.getElementById(3).value=A1;"));
  client.println(F("  var k1=bigInt(B1).modPow(a,p);"));
  client.println(F("  k1string = new String(k1.toString());"));

  client.println(F("var longi=k1string.length;"));
  client.println(F("for (i=0;i<(16-longi);i++) {"));
  client.println(F("  k1string = k1string+k1string.charAt(i);"));
  client.println(F("}"));

  client.println(F("usuario=input_user.value;"));
  client.println(F("pass=input_pass.value;"));
  client.println(F("var longuser=usuario.length;"));
  client.println(F("var longpass=pass.length;"));

  client.println(F("for (i=0;i<(16-longuser);i++) {"));
  client.println(F("  usuario = usuario+\"~\";"));
  client.println(F("}"));

  client.println(F("for (i=0;i<(16-longpass);i++) {"));
  client.println(F("  pass = pass+\"~\";"));
  client.println(F("}"));

  client.println(F("var k1Bytes= aesjs.utils.utf8.toBytes(k1string);"));
  client.println(F("var usuarioBytes= aesjs.utils.utf8.toBytes(usuario);"));
  client.println(F("var passBytes= aesjs.utils.utf8.toBytes(pass);"));

  client.println(F("var aesEcb = new aesjs.ModeOfOperation.ecb(k1Bytes);"));
  client.println(F("var usuario_encryptedBytes = aesEcb.encrypt(usuarioBytes);"));
  client.println(F("var pass_encryptedBytes = aesEcb.encrypt(passBytes);"));

  //client.println(F("var usuario_encryptedHex = aesjs.utils.hex.fromBytes(usuario_encryptedBytes);"));
  //client.println(F("var pass_encryptedHex = aesjs.utils.hex.fromBytes(pass_encryptedBytes);"));

  client.println(F("input_user.type=\"password\";"));
   
  client.println(F("input_user.value=usuario_encryptedBytes;")); 
  client.println(F("input_pass.value=pass_encryptedBytes;"));
  

  client.println(F("}"));
  
  
  client.println(F("</script>"));

}


/////////////////////////////////////////////////////////////////////////////////////////////////
// Enviar código HTML. Parte de la cabecera del cuerpo de la página.
/////////////////////////////////////////////////////////////////////////////////////////////////
void imprimirCabeceraPagina(EthernetClient client){
  client.println(F("<body>"));
 
  client.println(F("<div class=\"view\">"));
  client.println(F("    <div class=\"header-wrapper\">"));
  client.println(F("      <h1>TFG. UOC. Sistema dom&oacute;tico <br> inal&aacute;mbrico con portal web.</h1>"));
  client.println(F("    </div>"));

}

/////////////////////////////////////////////////////////////////////////////////////////////////
// Enviar código HTML. Cuerpo de la página.
/////////////////////////////////////////////////////////////////////////////////////////////////
void imprimirPagina(byte tipo, EthernetClient client){

  client.println(F("<div  class=\"group-wrapper\">"));
  switch (tipo) {
    case 9 : // fallo autentificacion
      client.println(F("    <h2>Ha ocurrdio alg&uacute;n error. Vuelva a identificarse..</h2>"));

      
    case AUTORIZAR : // Identificacion
      client.println(F("    <h2>Por favor, identif&iacute;quese.</h2>"));
      client.println(F("<FORM>"));
      client.println(F("<table border=\"0\" align=\"center\">"));
      client.println(F("<tr>"));
      client.println(F("<td><h4>Usuario: </h4></td>"));
      client.println(F("<td></td><td><INPUT maxlength=\"15\" TYPE=\"text\" id=1 NAME=\"user\"</td>"));
      client.println(F("<td></td>"));
      client.println(F("</tr>"));
      client.println(F("<tr>"));
      client.println(F("<td><h4>Contrase&ntilde;a: </h4></td>"));
      client.println(F("<td></td><td><INPUT maxlength=\"15\" TYPE=\"password\" id=2 NAME=\"pass\"</td>"));
      client.println(F("</tr>"));
      client.println(F("<tr>"));
      client.println(F("<input type=\"hidden\" id=3 name=\"A1\" value=\"A1\">"));
      client.println(F("  <td> <input class=\"sergio\" onclick=\"diffie()\" type=\"submit\" value=\"Acceder\"></td>"));
      client.println(F("</tr>"));
      client.println(F("</table>"));
      client.println(F("</FORM>"));
      break;

    case CAMBIOPASSWORD : // Identificacion
      client.println(F("    <h2>Cambio de datos de acceso.</h2>"));
      client.println(F("    <h2>Introduzca los nuevos datos.</h2>"));
      client.println(F("<FORM>"));
      client.println(F("<table border=\"0\" align=\"center\">"));
      client.println(F("<tr>"));
      client.println(F("<td><h4>Usuario: </h4></td>"));
      client.println(F("<td></td><td><INPUT maxlength=\"15\" TYPE=\"text\" id=1 NAME=\"newuser\"</td>"));
      client.println(F("<td></td>"));
      client.println(F("</tr>"));
      client.println(F("<tr>"));
      client.println(F("<td><h4>Contrase&ntilde;a: </h4></td>"));
      client.println(F("<td></td><td><INPUT maxlength=\"15\" TYPE=\"password\" id=2 NAME=\"newpass\"</td>"));
      client.println(F("</tr>"));
      client.println(F("<tr>"));
      client.println(F("<input type=\"hidden\" id=3 name=\"A1\" value=\"A1\">"));
      client.println(F("  <td> <input class=\"sergio\" onclick=\"diffie()\" type=\"submit\" value=\"Cambiar\"></td>"));
      client.println(F("</tr>"));
      client.println(F("</table>"));
      client.println(F("</FORM>"));
      break;
    
    case MENU: // Menu elementos
      client.println(F("    <h2>Elija una opci&oacute;n.</h2>"));
      client.println(F("<FORM>"));
      client.println(F("<table border=\"0\" align=\"center\">"));
      for (int i=0;i < numElementos
      ;i++) {
        client.println(F("<tr>"));
        client.print(F("<td><button class=\"sergio\" type=\"button\" style=\"width:120px; align:center\" name\""));
        client.print(elementos[i]);
        if (elementos[i]==elementos[CAMBIOPASSWORD]){
          client.print(F("\" onClick=\"parent.location='/?"));
        } else {
          client.print(F("\" onClick=\"parent.location='/?elemento="));
        }
        
        client.print(elementos[i]);
        client.print(F("'\">"));
        client.print(elementos[i]);
        client.println(F("</button></td>"));
        client.println(F("</tr>"));
      }
      client.println(F("<tr>"));
      client.print(F("<td><button class=\"sergio\" type=\"button\" style=\"width:120px; align:center\" name\""));
      client.print(F("Salir"));
      client.print(F("\" onClick=\"parent.location='/DomosSalir"));
      client.print(F("'\">"));
      client.print(F("Salir"));
      client.println(F("</button></td>"));
      client.println(F("</tr>"));
      client.println(F("</table>"));
      client.println(F("</FORM>"));
      break;
    case LUCES : //luces
      
      client.println(F("    <h2>Luces</h2>"));
      client.println(F("<FORM>"));
      client.println(F("<table border=\"0\" align=\"center\">"));
      for (int i=0;i < maxLuces ;i++) {
        client.println(F("<tr>"));
        client.print(F("<td><h4>"));
        client.print(i+1);
        client.print(F(". "));
        client.print(luces[i].getEstancia());
        client.println(F("</h4></td>"));
        client.print("<td><INPUT TYPE=\"button\" VALUE=\"ON \" onClick=\"parent.location='/?"+ elementos[tipo]+"=H");
        client.print(luces[i].getEstancia());
        client.print(F("'\">"));
        client.println(F("</td>"));
        client.print("<td><INPUT TYPE=\"button\" VALUE=\"OFF \" onClick=\"parent.location='/?"+ elementos[tipo]+"=L");
        client.print(luces[i].getEstancia());
        client.print(F("'\">"));
        client.println(F("</td>"));
        if (luces[i].getEstado()==true) {
          client.println(F("<td><div class='green-circle'><div class='glare'></div></div></td>"));
        } else {
          client.println(F("<td><div class='black-circle'><div class='glare'></div></div></td>"));
        }
        
        client.println(F("</tr>"));
      }

      break;
      
      case  TEMPERATURA :
      client.println(F("    <h2>Temperaturas</h2>"));
      client.println(F("<FORM>"));
      client.println(F("<table border=\"0\" align=\"center\">"));
      for (int i=0;i < maxTemperaturas ;i++) {
        client.println(F("<tr>"));
        client.print(F("<td><h4>"));
        client.print(i+1);
        client.print(F(". "));
        client.print(temperaturas[i].getEstancia());
        client.println(F("</h4></td>"));
        client.println(F("<td></td>"));
        client.print(F("<td><h4> Temperatura: "));
        client.print(temperaturas[i].getValor());
        client.println(F(" &#176 C</h4></td>"));
               
        client.println(F("</tr>"));
      }

      break;

      case PERSIANAS :
      client.println(F("    <h2>Persianas</h2>"));
      client.println(F("<FORM>"));
      client.println(F("<table border=\"0\" align=\"center\">"));
      for (int i=0;i < maxPersianas ;i++) {
        client.println(F("<tr>"));
        client.print(F("<td><h4>"));
        //client.print(i+1);
        //client.print(F(". "));
        client.print(persianas[i].getEstancia());
        client.println(F("</h4></td>"));
        client.print("<td><INPUT TYPE=\"button\" VALUE=\"Bajada \" onClick=\"parent.location='/?"+ elementos[tipo]+"=0");
        client.print(persianas[i].getEstancia());
        client.print(F("'\">"));
        client.println(F("</td>"));
        client.print("<td><INPUT TYPE=\"button\" VALUE=\"Un cuarto \" onClick=\"parent.location='/?"+ elementos[tipo]+"=1");
        client.print(persianas[i].getEstancia());
        client.print(F("'\">"));
        client.println(F("</td>"));
        client.print("<td><INPUT TYPE=\"button\" VALUE=\"A mitad \" onClick=\"parent.location='/?"+ elementos[tipo]+"=2");
        client.print(persianas[i].getEstancia());
        client.print(F("'\">"));
        client.println(F("</td>"));
        client.print("<td><INPUT TYPE=\"button\" VALUE=\"Tres cuartos \" onClick=\"parent.location='/?"+ elementos[tipo]+"=3");
        client.print(persianas[i].getEstancia());
        client.print(F("'\">"));
        client.println(F("</td>"));
        client.print("<td><INPUT TYPE=\"button\" VALUE=\"Subida \" onClick=\"parent.location='/?"+ elementos[tipo]+"=4");
        client.print(persianas[i].getEstancia());
        client.print(F("'\">"));
        client.println(F("</td>"));
        
        switch (persianas[i].getEstado()) {
          case bajada:
            client.println(F("<td><h4>Persiana bajada</h4></td>"));
            
          break;
          case un_cuarto:
            client.println(F("<td><h4>Persiana subida 1/4</h4></td>"));

          break;
          case mitad:
            client.println(F("<td><h4>Persiana a mitad</h4></td>"));

          break;
          case tres_cuartos:
            client.println(F("<td><h4>Persiana subida a 3/4</h4></td>"));

          break;
          case subida:
            client.println(F("<td><h4>Persiana subida totalmente</h4></td>"));

          break;
        }
        
        client.println(F("</tr>"));
      }
    
      
      break;   

      case AACC :
      client.println(F("    <h2>AACC.</h2>"));
      client.println(F("<FORM>"));
      client.println(F("<table border=\"0\" align=\"center\">"));
      client.println(F("<tr>"));
      client.print(F("<td><h4> Estado: "));
      switch (aire.getEstado()) {
          case apagado:
            client.println(F("<td>apagado.</td>"));
            
          break;
          case encendido:
            client.println(F("<td>encendido</td>"));

          break;
          case automatico:
            client.println(F("<td>automatico</td>"));

          break;
          case cool:
            client.println(F("<td>cool</td>"));

          break;
          case dry:
            client.println(F("<td>dry</td>"));

          break;
          case fan:
            client.println(F("<td>fan</td>"));
            switch (aire.getFan()) {
              case 0:
                client.println(F("<td>al minimo</td>"));

              break;
              case 1:
                client.println(F("<td>a mitad de potencia</td>"));

              break;
              case 2:
                client.println(F("<td>a 3/4 de potencia</td>"));

              break;
              case 3:
                client.println(F("<td>al maximo</td>"));

              break;           
            }
            

          break;
          case heat:
            client.println(F("<td>heat</td>"));

          break;
          case quiet:
            client.println(F("<td>quiet</td>"));

          break;
        }
      client.println(F("</h4></td></tr>"));
      client.print(F("<tr><td><h4> Temperatura: <td>")); 
      client.print(aire.getTemperatura());
      client.println(F("</td></h4></td></tr>"));

      client.println(F("<tr>"));
      client.println("<td><INPUT TYPE=\"button\" VALUE=\"Apagar \" style=\"width:120px; align:center\" onClick=\"parent.location='/?AACC=0'\"></td>");
      client.println(F("</tr>"));
      client.println(F("<tr>"));
      client.println("<td><INPUT TYPE=\"button\" VALUE=\"Encender \" style=\"width:120px; align:center\" onClick=\"parent.location='/?AACC=1'\"></td>");
      client.println(F("</tr>"));
      client.println(F("<tr>"));
      client.println("<td><INPUT TYPE=\"button\" VALUE=\"Automatico \" style=\"width:120px; align:center\" onClick=\"parent.location='/?AACC=2'\"></td>");
      client.println(F("</tr>"));
      client.println(F("<tr>"));
      client.println("<td><INPUT TYPE=\"button\" VALUE=\"Cool \" style=\"width:120px; align:center\" onClick=\"parent.location='/?AACC=3'\"></td>");
      client.println(F("</tr>"));
      client.println(F("<tr>"));
      client.println("<td><INPUT TYPE=\"button\" VALUE=\"Dry \" style=\"width:120px; align:center\" onClick=\"parent.location='/?AACC=4'\"></td>");
      client.println(F("</tr>"));
      client.println(F("<tr>"));
      client.println("<td><INPUT TYPE=\"button\" VALUE=\"Fan \" style=\"width:120px; align:center\" onClick=\"parent.location='/?AACC=5'\"></td>");
      client.println(F("</tr>"));
      client.println(F("<tr>"));
      client.println("<td><INPUT TYPE=\"button\" VALUE=\"Heat \" style=\"width:120px; align:center\" onClick=\"parent.location='/?AACC=6'\"></td>");
      client.println(F("</tr>"));
      client.println(F("<tr>"));
      client.println("<td><INPUT TYPE=\"button\" VALUE=\"Quiet \" style=\"width:120px; align:center\" onClick=\"parent.location='/?AACC=7'\"></td>");
      client.println(F("</tr>"));
      client.println(F("<tr>"));
      client.println("<td><INPUT TYPE=\"button\" VALUE=\"Subir temperatura \" style=\"width:120px; align:center\" onClick=\"parent.location='/?AACC=8'\"></td>");
      client.println(F("</tr>"));
      client.println(F("<tr>"));
      client.println("<td><INPUT TYPE=\"button\" VALUE=\"Bajar temperatura \" style=\"width:120px; align:center\" onClick=\"parent.location='/?AACC=9'\"></td>");
      client.println(F("</tr>"));
      
           
      break;

      case PUERTAS :
      client.println(F("    <h2>Puertas</h2>"));
      client.println(F("<FORM>"));
      client.println(F("<table border=\"0\" align=\"center\">"));
      for (int i=0;i < maxPuertas ;i++) {
        client.println(F("<tr>"));
        client.print(F("<td><h4>"));
        client.print(i+1);
        client.print(F(". "));
        client.print(puertas[i].getEstancia());
        client.println(F("</h4></td>"));
        client.println(F("<td></td>"));
        if (puertas[i].getEstado()==true) {
          client.println(F("<td><h4>Puerta abierta</h4></td>"));
        } else {
          client.println(F("<td><h4>Puerta cerrada</h4></td>"));
        }
        
        client.println(F("</tr>"));
      }
      
      
      break;   
      
      case PAGINA_CONSUMO : 
      client.println(F("    <h2>Consumos</h2>"));
      client.println(F("<FORM>"));
      client.println(F("<table border=\"0\" align=\"center\">"));
      for (int i=0;i < maxConsumos ;i++) {
        client.println(F("<tr>"));
        client.print(F("<td><h4>"));
        client.print(i+1);
        client.print(F(". "));
        client.print(consumos[i].getEstancia());
        client.println(F("</h4></td>"));
        client.println(F("<td></td>"));
        client.print(F("<td><h4> Consumo "));
        client.print(consumos[i].getValor());
        client.println(F(" W</h4></td>"));
               
        client.println(F("</tr>"));
      }
  
      break;   

    
  }
  if (tipo==LUCES or tipo==TEMPERATURA or tipo==PERSIANAS or tipo ==AACC or tipo==PAGINA_CONSUMO or tipo==PUERTAS){
    client.println(F("<tr></tr><table> <tr>"));
    client.print(F("<td><button class=\"sergio\" type=\"button\" style=\"width:120px; align:center\" name\""));
    client.print(F("Salir"));
    client.print(F("\" onClick=\"parent.location='/DomosSalir"));
    client.print(F("'\">"));
    client.print(F("Salir"));
    client.println(F("</button></td>"));
    client.print(F("<td><button class=\"sergio\" type=\"button\" style=\"width:120px; align:center\" name\""));
    client.print(F("Volver"));
    client.print(F("\" onClick=\"parent.location='/DomosVolver"));
    client.print(F("'\">"));
    client.print(F("Volver"));
    client.println(F("</button></td>"));
    client.println(F("</tr>"));
    client.println(F("</table></table>"));
    client.println(F("</FORM>"));
  }
  
  client.println(F("<h3 align=\"center\"> Alumno - Sergio Fern&aacute;ndez Ferri <br><br> Grado de Tecnolog&iacute;a de Telecomunicaci&oacute;n <br><br> Trabajo Final de Grado - Junio 2017 </h3>"));
  client.println(F("</div>"));
  client.println(F("</div>"));
  client.println(F("</body>"));
  
}

/////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////
// Función principal del programa que se ejecutará como un bucle infinito en el microcontrolador
/////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////

void loop() {

  //Actualizamos el estado de la red RF

  if(millis()-tiempoDDNS > MAXTIEMPOACTUALIZACIONIP) {
    actualizaIPDNS();
    tiempoDDNS=millis();
  }

  network.update();
  
  EthernetClient client = server.available(); // creamos un socket para comunicación con el cliente
  
  lectura=checkForClient(client); // Comprobamos si algún cliente ha hecho una petición al servidor y la guardamos en la variable de lectura.

  if ( (lectura.substring(lectura.indexOf("GET /")+5,lectura.indexOf("GET /")+13)) == "Domos HT") {
    
    autentificar(client);
        
    //delay(5);
    
  } else if ( (lectura.substring(lectura.indexOf("?user")+5,lectura.indexOf("?user")+6)) == "=") {
    
    if (autorizado=comprobarAutentificacion(client)) {
        actualizarTiempoActividadAutorizada();
        menuFuncionesHtml(client);
      } else {
        mensajeNoaut(client);
      }
    
    
  } else if (( (lectura.substring(lectura.indexOf("GET /")+5,lectura.indexOf("GET /")+14)) == "?elemento")) {
    if (autorizado && comprobarTiempoInactivo()) {
      // Seleccion de elemento
      lectura=lectura.substring(lectura.indexOf("/?elemento=")+11,lectura.indexOf(" HTTP"));
      //Serial.println(lectura);
      
      for (byte i=0 ; i<numElementos;i++ ){
        if (lectura == elementos[i]){ 
        //Serial.print("Entrando en : ");
        //Serial.println(elementos[i]);
        recogerInfo(i);
        imprimirCabeceraHtml(REFRESCAR, i, client);
        imprimirCSS(i, client);
        imprimirCabeceraPagina(client);
        imprimirPagina(i, client); 
        }
      }
      //actualizarTiempoActividadAutorizada();
    } else {
      mensajeNoaut(client);
    }
    
  }  else  if (( (lectura.substring(lectura.indexOf("GET /")+5,lectura.indexOf("GET /")+15)) == "DomosSalir")) {
      //Serial.print("Saliendo.");
  
      autorizado=false;
  
      autentificar(client);
     
  } 
  else if (( (lectura.substring(lectura.indexOf("GET /")+5,lectura.indexOf("GET /")+16)) == "DomosVolver")) {
    if (autorizado && comprobarTiempoInactivo()) {
      menuFuncionesHtml(client);
      actualizarTiempoActividadAutorizada();
    } else {
      mensajeNoaut(client);
    }
     
  }

  else if (( (lectura.substring(lectura.indexOf("GET /?")+6,lectura.indexOf("GET /?")+6+elementos[LUCES].length())) == elementos[LUCES])) {
    
    if (autorizado && comprobarTiempoInactivo()) {
      if ( lectura.substring(lectura.indexOf(elementos[LUCES])+elementos[LUCES].length()+1,lectura.indexOf(elementos[LUCES])+elementos[LUCES].length()+2) == "H") {
        lectura=lectura.substring(lectura.indexOf(elementos[LUCES])+elementos[LUCES].length()+2,lectura.indexOf(" HTT"));
        lectura.replace("%20"," ");
        encender(lectura);
        
        
      } else if ( ( (lectura.substring(lectura.indexOf(elementos[LUCES])+int(sizeof(elementos[LUCES])),lectura.indexOf(elementos[LUCES])+int(sizeof(elementos[LUCES]))+1)) == "L") ) {
        lectura=lectura.substring(lectura.indexOf(elementos[LUCES])+elementos[LUCES].length()+2,lectura.indexOf(" HTT"));
        lectura.replace("%20"," ");
        apagar(lectura);    
        
      }
    
      imprimirCabeceraHtml(REFRESCAR, LUCES, client);
      imprimirCSS(LUCES, client);
      imprimirCabeceraPagina(client);
      imprimirPagina(LUCES, client);
      
      actualizarTiempoActividadAutorizada();
    
    } else {
      mensajeNoaut(client);
    }
     
  }
  else if (( (lectura.substring(lectura.indexOf("GET /?")+6,lectura.indexOf("GET /?")+6+elementos[PERSIANAS].length())) == elementos[PERSIANAS])) {
    //Serial.println("Persianas");
    //Serial.println(elementos[PERSIANAS].length());
    
    if (autorizado && comprobarTiempoInactivo()) {

      // Bajar persiana del todo
      if ( lectura.substring(lectura.indexOf(elementos[PERSIANAS])+elementos[PERSIANAS].length()+1,lectura.indexOf(elementos[PERSIANAS])+elementos[PERSIANAS].length()+2) == "0") {
        lectura=lectura.substring(lectura.indexOf(elementos[PERSIANAS])+elementos[PERSIANAS].length()+2,lectura.indexOf(" HTT"));
        lectura.replace("%20"," ");
        //Serial.println("Ahora");
        persiana(lectura, bajada);
        
      } else if ( lectura.substring(lectura.indexOf(elementos[PERSIANAS])+elementos[PERSIANAS].length()+1,lectura.indexOf(elementos[PERSIANAS])+elementos[PERSIANAS].length()+2) == "1") {
        lectura=lectura.substring(lectura.indexOf(elementos[PERSIANAS])+elementos[PERSIANAS].length()+2,lectura.indexOf(" HTT"));
        lectura.replace("%20"," ");
        persiana(lectura, un_cuarto);    
      
      } else if ( lectura.substring(lectura.indexOf(elementos[PERSIANAS])+elementos[PERSIANAS].length()+1,lectura.indexOf(elementos[PERSIANAS])+elementos[PERSIANAS].length()+2) == "2") {
        lectura=lectura.substring(lectura.indexOf(elementos[PERSIANAS])+elementos[PERSIANAS].length()+2,lectura.indexOf(" HTT"));
        lectura.replace("%20"," ");
        persiana(lectura, mitad);    
      
      } else if ( lectura.substring(lectura.indexOf(elementos[PERSIANAS])+elementos[PERSIANAS].length()+1,lectura.indexOf(elementos[PERSIANAS])+elementos[PERSIANAS].length()+2) == "3") {
        lectura=lectura.substring(lectura.indexOf(elementos[PERSIANAS])+elementos[PERSIANAS].length()+2,lectura.indexOf(" HTT"));
        lectura.replace("%20"," ");
        persiana(lectura, tres_cuartos);  
         
      } else if ( lectura.substring(lectura.indexOf(elementos[PERSIANAS])+elementos[PERSIANAS].length()+1,lectura.indexOf(elementos[PERSIANAS])+elementos[PERSIANAS].length()+2) == "4") {
        lectura=lectura.substring(lectura.indexOf(elementos[PERSIANAS])+elementos[PERSIANAS].length()+2,lectura.indexOf(" HTT"));
        lectura.replace("%20"," ");
        persiana(lectura, subida); 
          
      }
      
      imprimirCabeceraHtml(REFRESCAR, PERSIANAS, client);
      imprimirCSS(PERSIANAS, client);
      imprimirCabeceraPagina(client);
      imprimirPagina(PERSIANAS, client);

      actualizarTiempoActividadAutorizada();
      
    } else {
      mensajeNoaut(client);
    }
  }

  else if (( (lectura.substring(lectura.indexOf("GET /?")+6,lectura.indexOf("GET /?")+6+elementos[AACC].length())) == elementos[AACC])) {
    //Serial.println("AACC12");
    if (autorizado && comprobarTiempoInactivo()) {

      switch (lectura.substring(lectura.indexOf(elementos[AACC])+elementos[AACC].length()+1,lectura.indexOf(elementos[AACC])+elementos[AACC].length()+2).toInt()) {
        
        case 0:
        //Serial.println("Pues funciona 0");
    

        break;
        case 1:
        //Serial.println("Pues funciona 1");
        

        break;
        case 2:
        //Serial.println("Pues funciona 2");
      

        break;
        case 3:
        //Serial.println("Pues funciona 3");
        

        break;
        case 4:
        //Serial.println("Pues funciona 4");
     

        break;
        case 5:
        //Serial.println("Pues funciona 5");
        

        break;
        case 6:
        //Serial.println("Pues funciona 6");
        

        break;
        case 7:
        //Serial.println("Pues funciona 7");
       

        break;
        case 8:
        //Serial.println("Pues funciona 8");
        

        break;
        case 9:
        //Serial.println("Pues funciona 9");
   

        break;
      }
      

      imprimirCabeceraHtml(REFRESCAR, AACC, client);
      imprimirCSS(AACC, client);
      imprimirCabeceraPagina(client);
      imprimirPagina(AACC, client);

      actualizarTiempoActividadAutorizada();
      
    } else {
      mensajeNoaut(client);
    }
  } else if (( (lectura.substring(lectura.indexOf("GET /?")+6,lectura.indexOf("GET /?")+6+7)) == "Cambiar")) {
    if (autorizado && comprobarTiempoInactivo()) {
      //Serial.println("Cambiar");
      paginaCambioDatosAcceso(client);
    } else {
      mensajeNoaut(client);
    }
    
    
  } else if ( (lectura.substring(lectura.indexOf("?newuser")+8,lectura.indexOf("?newuser")+9)) == "=") {
    
    if (autorizado=cambioDatosAcceso(client)) {
        actualizarTiempoActividadAutorizada();
        menuFuncionesHtml(client);
      } else {
        mensajeNoaut(client);
      }
    }  
    
     
  

   
  delay(1);  
  client.stop();
  
  // Leemos la temperatura continuamente para hacer pasar corriente por el LM35DZ (en su pin de salida hacia Arduino) para que la toma de temperatura sea más precisa.
  //leerTemperatura();
  

}

