
 /** Trabajo Final de Grado. UOC.
  *  11 de junio del 2017
  *  Sergio Fernández Ferri
  *  Grado en Tecnología de las comunicaciones. Esp. Sistemas Telecomunicación.
  *  -------------------------------------------------------------------------
  *  Sistema Domótico Arduino con RF y portal WEB.
  *  -------------------------------------------------------------------------
  *  Código a implementar en los nodos remotos de la vivienda.
  *  -------------------------------------------------------------------------
  */

#include "RF24.h"
#include "RF24Network.h"
//#include <SPI.h>

/********************************/
/**** Definimos unos códigos para la comunicación de los módulos RF ****/
/**** Tipo de consultas ****/
#define ESTADO_LUZ 1001
#define ESTADO_PUERTA 1002
#define VER_CONSUMO 1003
#define VER_TEMPERATURA 1004
#define ESTADO_PERSIANA 1005
#define ESTADO_AACC 1006

/**** Tipo de acciones ****/
#define APAGAR_LUZ 1201
#define ENCENDER_LUZ 1211

#define BAJAR_PERSIANA 1215
#define SUBIR_PERSIANA 1225

#define ENCEDER_AACC 1216
#define APAGAR_AACC 1226
#define SUBIR_TEMP_AACC 1236
#define BAJAR_TEMP_AACC 1246
#define FAN_AACC 1256
/********************************/

/**** Creación del objeto de control del módulo NRF24lL01. Los parámetros son el pin CE y CS ****/
RF24 radio(9, 10);

/**** Creación del objeto que implementa la capa de red con módulo RF. Le pasamos como parámetro el objeto del módulo RF ****/
RF24Network network(radio);

/**** Dirección de red del nodo según la estructura del diseño. Al nodo del dormitorio 3 le corresponde la dirección 031. ****/
const uint16_t this_node=031; // Modificar según nodo. 

/****************************************************************************/
/**** Clase Luz que representa el funionamiento de la luz de la estancia ****/
class Luz {
  private:
    boolean Estado; // Representa si la luz está encendida o apgada.
    byte pin_control; // Representa el interruptor. El pin que actuará sobre el relé (pin digital).

  public: // Métodos de la clase Luz.
    Luz(byte pin); // Para crear un objeto luz debemos de indicar sobre que pin debe actuar el microcontrolador.
    void Encender(); // Método que realizará las acciones oportunas para encender la luz.
    void Apagar(); // Método que realizará las acciones oportunas para apagar la luz.
    void setPincontrol(byte pin); // Sirve para fijar o modifcar el pin que se utilizará para gobernar el relé.
    void setEstado(boolean estado); // Para fijar o modificar el valor de la variable Estado.
    boolean getEstado(); // Nos devolverá el valor de la variable Estado.
    byte getPincontrol(); // Nos devolverá el pin de control.
    
};

/**** Constructor de la clase Luz ****/
Luz::Luz(byte pin) {
  pin_control=pin; 
  pinMode(pin_control, OUTPUT); // Configuramos el pin digital de control como OUTPUT.
}

/**** Método que enciende la luz ****/
void Luz::Encender() {
  // Actuar sobre el relé para cambiar el estado solo si la luz está apagada (!Estado).
  Estado=getEstado(); // Almacenamos el estado después antes para comprobar como está la luz.
  if (!Estado) {
    if (digitalRead(pin_control)) { // Como actuaremos sobre un conmutador debermos cambiar la alimentación actual del relé
       digitalWrite(pin_control,LOW); // Si actualmente estamos activando el relé, dejaremos de activarlo, y viceversa.
    } else {
      digitalWrite(pin_control,HIGH);
    }
  }
}

/**** Método para apagar la luz ****/
void Luz::Apagar() {
  // Actuar sobre el relé para cambiar el estado solo si la luz está encendida (Estado).
  Estado=getEstado(); // Almacenamos el estado antes del cambio para comprobar la luz.
    if (Estado) {
    if (digitalRead(pin_control)) { // Como actuaremos sobre un conmutador debermos cambiar la alimentación actual del relé
       digitalWrite(pin_control,LOW); // Si actualmente estamos activando el relé, dejaremos de activarlo, y viceversa.
    } else {
      digitalWrite(pin_control,HIGH);
    }
  } 
}

/**** Método para fijar o modificar el pin de control. Indicamo que el pin de control será una salida del microcontrolador. ****/
void Luz::setPincontrol(byte pin) {pin_control=pin; pinMode(pin_control, OUTPUT);}

/**** Método para fijar el estado de la luz ****/
void Luz::setEstado(boolean estado) {Estado=estado;}

/**** Método para obtener el estado de la luz. Utilizamos el sensor de corriente ACS712 para comprobar el estado de la luz ****/
boolean Luz::getEstado() {
  // Verificar consumo en línea de luz para dar el estado mediante el sensor de corriente
  float voltajeSensor;
  float corriente=0;
  long tiempo=millis();
  float Imax=0;
  float Imin=0;
  
  while(millis()-tiempo<15)//realizamos mediciones durante 10 milisegundos para asegurarnos del estado de la luz.
  { 
    voltajeSensor = analogRead(/* pin analógico dónde esté conectado el ACS712 detector de corriente asociado a la luz */ A1) * (5.0 / 1023.0);//lectura del sensor
    corriente=0.9*corriente+0.1*((voltajeSensor-2.5)/0.066); //Ecuación  para obtener la corriente
    if(corriente>Imax)Imax=corriente;
    if(corriente<Imin)Imin=corriente;
  }
  float consumo=(((Imax-Imin)/2)*1.20-0.045);
  
  if (consumo > 0.1) {Estado=true;} // Si hay consumo la luz está encendidad.
  else {Estado=false;} // En caso contrario apagada.
    
  return false; // cambiar a "Estado"
}

/**** Método para obtener el pin de control ****/
byte Luz::getPincontrol() {return pin_control;}


/***************************************************************************************/
/***************************************************************************************/

/**** Función Consumo() que nos devolverá el consumo ****/

float Consumo() {
  
  float Sensibilidad=0.066; //sensibilidad en V/A para nuestro sensor (para el modelo ACS712 ELC 30)
  float offset=0.08; // Equivale a la amplitud del ruido (Obtenida experimentalmente mediante varias mediciones para calibrarlo).

  float voltajeSensor; // Para guardar el voltaje que nos da el sensor.
  byte pin_sensor=2; // Pin analógico donde tenemos la salida del sensor ACS712. En este es el 2.
  float corriente=0; // Para guardar el valor de la corriente (transformándo el valor de la tensión según la relación del fabricante del sensor).
  long tiempo=millis(); // PAra contar el tiempo y realizar varias medidas en un tiempo determinado.
  float Imax=0; // Para guardar el valor máximo de la corriente
  float Imin=0; // Para guardar el valor mínimo de la corriente
  
  while(millis()-tiempo<50)//realizamos mediciones durante 50 milisegundos. Teniendo en cuenta una red 220 V 50 Hz, en la que cada ciclo dura 20 ms, en 50 ms tomaremos medidas de 2.5 ciclos.
  { 
    voltajeSensor = analogRead(pin_sensor) * (5.0 / 1023.0);//lectura del sensor con la fórmula que indica el fabricante.
    corriente=0.9*corriente+0.1*((voltajeSensor-2.5)/Sensibilidad); //Ecuación  para obtener la corriente. Es un promedio.
    if(corriente>Imax)Imax=corriente;  // Almacenamos el valor máximo
    if(corriente<Imin)Imin=corriente;  // y mínimo de la corriente
  }
  
  float Ip=((Imax-Imin)/2)*1.20-offset;//obtenemos la corriente pico. El "1.20" es un factor correcto obtenido experimentalmente.
  float Irms=Ip*0.707; //Intensidad RMS (Eficaz) = Ipico/(2^1/2)
  float P=Irms*220.0; // P=IV Watios
  
  return(P);
}

/**** Función que nos devuelve la temperatura del sensor LM35DZ ****/
float leerTemperatura() {
    byte pin_sensor=1; // Pin en el que conectamos el sensor LM35. En este caso el 1.
    
  /**** Dado que el LM35DZ tiene un rango de valores desde -55ºC (correspndiente a -550mV) hasta 150ºC (correspondiente a 1500mV), y la temperatura ambiente puede rondar desde los -10ºC hasta los 45ºC (por ejemplo) tendremos valores entre -10mV y 450 mV.
   *  Por lo tanto podemos modificar el valor de referencia de la tensión analógica del microcontrolador para que la precisión sea mayor. 
   */
  analogReference(INTERNAL); // Para Arduino Nano
  int valorLM35 = analogRead(pin_sensor); // Leemos en el pin analógico 1.
  float milivoltios = (1100 /1023) * valorLM35; // Tensión en milivoltios. 1100 son los milivoltios de la tensión de referencia. 1023 es la resolución del conversor AD (10 bits ->1024 valores, del 0 al 1023)
  float temp=milivoltios/10;
  Serial.print("Temperatura local : ");
    
  Serial.println(temp);
  return temp;
}

/**** Función que nos devuelve el estado de la puerta ****/


boolean verEstadoPuerta() {
  byte pin_sensor = 4; // Pin en el que conectamos el sensor magnético. En este caso el 4 (pin digital).
  pinMode(pin_sensor, INPUT); // Configuramos el pin digital en modo INPUT.
  return(digitalRead(4)); // Leemos el pin digital 4 y devolvemos su estado ("1" o "0").
} 


Luz luz(/* pin en el que estará conectado el relé */3); // Creamos una instancia de la clase Luz (un objeto Luz). Actuará sobre el pin digital 3

float consumo; // Creamos una variable para almacenar el consumo.

float temp;  // Creamos una variable para almacenar la temperatura.

/**** Función setup() del programa - Se ejecuta una vez al arranque del microcontrolador ****/

void setup() {
 
  Serial.begin(115200); // Para debug
  
  //SPI.begin(); No es preciso.                                          
  
  radio.begin(); // Activamos el módulo RF.
  radio.setPALevel(RF24_PA_HIGH); // Seleccionamos que funcione a casi la máxima potencia. Por defecto funciona en el modo PA_MAX, con un mayot consumo e interferencia a otros sistemas 2.4 GHz.
  network.begin(/*channel*/ 90, /*node address*/ this_node );  // Inicamos la red RF en el canal 90 de los 2.4 GHz y configurando este nodo con al dirección this_node.

  // Líneas de debug.
  Serial.println("Hola");
  Serial.print("Nodo : ");
  Serial.println(this_node, OCT);
  
}

/**** Función principal del programa. Esta función se ejecuta constantemente en el microcontrolador ****/
void loop() {
  
  network.update(); // Comprobamos el estado de la red (direcciones facilitadas por el nodo master, y existencia de nodos hijos.
  
  unsigned int message=0; // Para almacener los mensajes recibidos del nodo master. Inicializamos a 0.
  
  while(network.available()){
    // Si tenemos un mensaje disponible para este nodo....
    // Creamos un objeto cabecera de un paquete para almacenar los datos de cabecera de un posible paquete.
    RF24NetworkHeader header;
                                                                         
    network.read(header,&message,sizeof(unsigned long));  // Leemos un paquete y almacenamos la cabecera en header, y el mensaje en message hasta un tamaño unsigned long

    // Líneas de debug.
    Serial.print("APP Received :");
    Serial.print(message);
    Serial.print(" from :");
    Serial.println(header.from_node);
    
    //delay(5); // Realizamos una pequeña pausa de 5ms para volver a leer otra posible cabecera. NO es preciso.

  }
  RF24NetworkHeader header(00,'t'); // Creamos una cabecera de mensaje (sobreescribiendo la variable creada anteriormente). Con destino el nodo 00 (master).
    
         switch (message) {  // En función del mesnsaje recibido del nodo master....
          case ESTADO_LUZ:  // Si recibimos el mensje ESTADO_LUZ (1001) -> ejecutamos...
          
            Serial.println("---------------------------------");
            Serial.print("Enviando respuesta de actualización de estado : ");
            message=luz.getEstado();  // Grabamos el estado de la luz en la variable message.
            Serial.println(message);
            network.write(header,&message,sizeof(message));  // Enviamos el mensaje (que contiene el estado de la luz) al nodo master.
          
          break;

          case ENCENDER_LUZ:  // Si recibimos el mensaje ENCENDER_LUZ
      
            luz.Encender();  // Encendemos la luz.
            Serial.println("---------------------------------");
            Serial.print("Encendiendo la luz. Mensaje : ");
            Serial.println(message);
            //network.write(header,&message,sizeof(unsigned long));
            break;

            case APAGAR_LUZ:  // Si recibimos el mensaje APAGAR_LUZ
      
            luz.Apagar();  // Apagamos la luz.
            Serial.println("---------------------------------");
            Serial.print("Apagando la luz. Mensaje : ");
            Serial.println(message);
            //network.write(header,&message,sizeof(unsigned long));
            break;
            
            
            case ESTADO_PUERTA:  // Si reibimos ESTADO_PUERTA
            Serial.println("---------------------------------");
            Serial.print("Estado de la puerta. Mensaje : ");
            Serial.println(message);
            message=verEstadoPuerta();  // Almacenamos el estado de la puerta en message.
            network.write(header,&message,sizeof(message)); // Y enviamos la variable message al nodo master.
            break;

            
            case VER_TEMPERATURA:  // Si recibimos VER_TEMPERATURA
            Serial.println("---------------------------------");
            Serial.print("Dando temperatura. Mensaje : ");
            Serial.println(message);
            temp=leerTemperatura(); // Almacenamos el valor de la temperatura en la variable temp
            network.write(header,&temp,sizeof(temp));  // Enviamos el valor de la temperatura almacenado en temp
            break;

            
            case VER_CONSUMO:   // Si recibimos VER_CONSUMO
            Serial.println("---------------------------------");
            Serial.print("Dando Consumo. Mensaje : ");
            Serial.println(message);
            consumo=Consumo();  // Almacenamos el valor de la consumo en la variable consumo
            network.write(header,&consumo,sizeof(consumo));  // Enviamos el valor del consumo almacenado en consumo
            break;
          
         }
         
         
}

